@@ -1,6 +1,7 @@
 /* Auxiliary vector support for GDB, the GNU debugger.
 
-   Copyright (C) 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -20,15 +21,10 @@
 #ifndef AUXV_H
 #define AUXV_H
 
+#include "target.h"
+
 /* See "include/elf/common.h" for the definition of valid AT_* values.  */
 
-
-/* Avoid miscellaneous includes in this file, so that it can be
-   included by nm-*.h for the procfs_xfer_auxv decl if that is
-   used in NATIVE_XFER_AUXV.  */
-struct target_ops;		/* Forward declaration.  */
-
-
 /* Read one auxv entry from *READPTR, not reading locations >= ENDPTR.
    Return 0 if *READPTR is already at the end of the buffer.
    Return -1 if there is insufficient buffer for a whole entry.
@@ -44,19 +40,11 @@ extern int target_auxv_parse (struct target_ops *ops,
 extern int target_auxv_search (struct target_ops *ops,
 			       CORE_ADDR match, CORE_ADDR *valp);
 
-/* Print the contents of the target's AUXV on the specified file. */
+/* Print the contents of the target's AUXV on the specified file.  */
 extern int fprint_target_auxv (struct ui_file *file, struct target_ops *ops);
 
-
-/* This function is called like a to_xfer_partial hook,
-   but must be called with TARGET_OBJECT_AUXV.
-   It handles access via /proc/PID/auxv, which is the common method.
-   This function is appropriate for doing:
-	   #define NATIVE_XFER_AUXV	procfs_xfer_auxv
-   for a native target that uses inftarg.c's child_xfer_partial hook.  */
-
-extern LONGEST procfs_xfer_auxv (struct target_ops *ops,
-				 int /* enum target_object */ object,
+extern LONGEST memory_xfer_auxv (struct target_ops *ops,
+				 enum target_object object,
 				 const char *annex,
 				 gdb_byte *readbuf,
 				 const gdb_byte *writebuf,
