@@ -1,6 +1,6 @@
 /* Dump-to-file commands, for GDB, the GNU debugger.
 
-   Copyright (c) 2002, 2005, 2007, 2008, 2009, 2010
+   Copyright (c) 2002, 2005, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    Contributed by Red Hat.
@@ -32,20 +32,11 @@
 #include "target.h"
 #include "readline/readline.h"
 #include "gdbcore.h"
+#include "cli/cli-utils.h"
 
 #define XMALLOC(TYPE) ((TYPE*) xmalloc (sizeof (TYPE)))
 
 
-char *
-skip_spaces (char *chp)
-{
-  if (chp == NULL)
-    return NULL;
-  while (isspace (*chp))
-    chp++;
-  return chp;
-}
-
 char *
 scan_expression_with_cleanup (char **cmd, const char *def)
 {
@@ -149,8 +140,8 @@ bfd_openw_with_cleanup (const char *filename, const char *target,
       if (!bfd_set_format (obfd, bfd_object))
 	error (_("bfd_openw_with_cleanup: %s."), bfd_errmsg (bfd_get_error ()));
     }
-  else if (*mode == 'a')	/* Append to existing file */
-    {	/* FIXME -- doesn't work... */
+  else if (*mode == 'a')	/* Append to existing file.  */
+    {	/* FIXME -- doesn't work...  */
       error (_("bfd_openw does not work with append."));
     }
   else
@@ -211,7 +202,9 @@ dump_bfd_file (const char *filename, const char *mode,
 					  | SEC_ALLOC
 					  | SEC_LOAD));
   osection->entsize = 0;
-  bfd_set_section_contents (obfd, osection, buf, 0, len);
+  if (!bfd_set_section_contents (obfd, osection, buf, 0, len))
+    warning (_("writing dump file '%s' (%s)"), filename, 
+	     bfd_errmsg (bfd_get_error ()));
 }
 
 static void
@@ -430,7 +423,7 @@ add_dump_command (char *name, void (*func) (char *args, char *mode),
     c->doc = concat ("Append ", c->doc + 6, (char *)NULL);
 }
 
-/* Opaque data for restore_section_callback. */
+/* Opaque data for restore_section_callback.  */
 struct callback_data {
   CORE_ADDR load_offset;
   CORE_ADDR load_start;
@@ -455,7 +448,7 @@ restore_section_callback (bfd *ibfd, asection *isec, void *args)
   gdb_byte *buf;
   int ret;
 
-  /* Ignore non-loadable sections, eg. from elf files. */
+  /* Ignore non-loadable sections, eg. from elf files.  */
   if (!(bfd_get_section_flags (ibfd, isec) & SEC_LOAD))
     return;
 
@@ -463,7 +456,7 @@ restore_section_callback (bfd *ibfd, asection *isec, void *args)
   if (sec_end <= data->load_start 
       || (data->load_end > 0 && sec_start >= data->load_end))
     {
-      /* No, no useable data in this section. */
+      /* No, no useable data in this section.  */
       printf_filtered (_("skipping section %s...\n"), 
 		       bfd_section_name (ibfd, isec));
       return;
@@ -474,7 +467,7 @@ restore_section_callback (bfd *ibfd, asection *isec, void *args)
      transfer should start and end.  */
   if (sec_start < data->load_start)
     sec_offset = data->load_start - sec_start;
-  /* Size of a partial transfer: */
+  /* Size of a partial transfer.  */
   sec_load_count -= sec_offset;
   if (data->load_end > 0 && sec_end > data->load_end)
     sec_load_count -= sec_end - data->load_end;
@@ -520,7 +513,11 @@ restore_binary_file (char *filename, struct callback_data *data)
 
   /* Get the file size for reading.  */
   if (fseek (file, 0, SEEK_END) == 0)
-    len = ftell (file);
+    {
+      len = ftell (file);
+      if (len < 0)
+	perror_with_name (filename);
+    }
   else
     perror_with_name (filename);
 
@@ -528,10 +525,10 @@ restore_binary_file (char *filename, struct callback_data *data)
     error (_("Start address is greater than length of binary file %s."), 
 	   filename);
 
-  /* Chop off "len" if it exceeds the requested load_end addr. */
+  /* Chop off "len" if it exceeds the requested load_end addr.  */
   if (data->load_end != 0 && data->load_end < len)
     len = data->load_end;
-  /* Chop off "len" if the requested load_start addr skips some bytes. */
+  /* Chop off "len" if the requested load_start addr skips some bytes.  */
   if (data->load_start > 0)
     len -= data->load_start;
 
@@ -551,7 +548,7 @@ restore_binary_file (char *filename, struct callback_data *data)
   if (fread (buf, 1, len, file) != len)
     perror_with_name (filename);
 
-  /* Now write the buffer into target memory. */
+  /* Now write the buffer into target memory.  */
   len = target_write_memory (data->load_start + data->load_offset, buf, len);
   if (len != 0)
     warning (_("restore: memory write failed (%s)."), safe_strerror (len));
@@ -573,7 +570,7 @@ restore_command (char *args, int from_tty)
   data.load_start  = 0;
   data.load_end    = 0;
 
-  /* Parse the input arguments.  First is filename (required). */
+  /* Parse the input arguments.  First is filename (required).  */
   filename = scan_filename_with_cleanup (&args, NULL);
   if (args != NULL && *args != '\0')
     {
@@ -586,18 +583,18 @@ restore_command (char *args, int from_tty)
 	  args += strlen (binary_string);
 	  args = skip_spaces (args);
 	}
-      /* Parse offset (optional). */
+      /* Parse offset (optional).  */
       if (args != NULL && *args != '\0')
       data.load_offset = 
 	parse_and_eval_address (scan_expression_with_cleanup (&args, NULL));
       if (args != NULL && *args != '\0')
 	{
-	  /* Parse start address (optional). */
+	  /* Parse start address (optional).  */
 	  data.load_start = 
 	    parse_and_eval_long (scan_expression_with_cleanup (&args, NULL));
 	  if (args != NULL && *args != '\0')
 	    {
-	      /* Parse end address (optional). */
+	      /* Parse end address (optional).  */
 	      data.load_end = parse_and_eval_long (args);
 	      if (data.load_end <= data.load_start)
 		error (_("Start must be less than end."));
@@ -617,10 +614,10 @@ restore_command (char *args, int from_tty)
     }
   else
     {
-      /* Open the file for loading. */
+      /* Open the file for loading.  */
       ibfd = bfd_openr_with_cleanup (filename, NULL);
 
-      /* Process the sections. */
+      /* Process the sections.  */
       bfd_map_over_sections (ibfd, restore_section_callback, &data);
     }
   return;
@@ -668,13 +665,13 @@ _initialize_cli_dump (void)
 {
   struct cmd_list_element *c;
 
-  add_prefix_cmd ("dump", class_vars, dump_command, _("\
-Dump target code/data to a local file."),
+  add_prefix_cmd ("dump", class_vars, dump_command,
+		  _("Dump target code/data to a local file."),
 		  &dump_cmdlist, "dump ",
 		  0/*allow-unknown*/,
 		  &cmdlist);
-  add_prefix_cmd ("append", class_vars, append_command, _("\
-Append target code/data to a local file."),
+  add_prefix_cmd ("append", class_vars, append_command,
+		  _("Append target code/data to a local file."),
 		  &append_cmdlist, "append ",
 		  0/*allow-unknown*/,
 		  &cmdlist);
@@ -689,32 +686,32 @@ Write the value of an expression to a raw binary file.\n\
 Arguments are FILE EXPRESSION.  Writes the value of EXPRESSION to\n\
 the specified FILE in raw target ordered bytes.");
 
-  add_prefix_cmd ("srec", all_commands, srec_dump_command, _("\
-Write target code/data to an srec file."),
+  add_prefix_cmd ("srec", all_commands, srec_dump_command,
+		  _("Write target code/data to an srec file."),
 		  &srec_cmdlist, "dump srec ", 
 		  0 /*allow-unknown*/, 
 		  &dump_cmdlist);
 
-  add_prefix_cmd ("ihex", all_commands, ihex_dump_command, _("\
-Write target code/data to an intel hex file."),
+  add_prefix_cmd ("ihex", all_commands, ihex_dump_command,
+		  _("Write target code/data to an intel hex file."),
 		  &ihex_cmdlist, "dump ihex ", 
 		  0 /*allow-unknown*/, 
 		  &dump_cmdlist);
 
-  add_prefix_cmd ("tekhex", all_commands, tekhex_dump_command, _("\
-Write target code/data to a tekhex file."),
+  add_prefix_cmd ("tekhex", all_commands, tekhex_dump_command,
+		  _("Write target code/data to a tekhex file."),
 		  &tekhex_cmdlist, "dump tekhex ", 
 		  0 /*allow-unknown*/, 
 		  &dump_cmdlist);
 
-  add_prefix_cmd ("binary", all_commands, binary_dump_command, _("\
-Write target code/data to a raw binary file."),
+  add_prefix_cmd ("binary", all_commands, binary_dump_command,
+		  _("Write target code/data to a raw binary file."),
 		  &binary_dump_cmdlist, "dump binary ", 
 		  0 /*allow-unknown*/, 
 		  &dump_cmdlist);
 
-  add_prefix_cmd ("binary", all_commands, binary_append_command, _("\
-Append target code/data to a raw binary file."),
+  add_prefix_cmd ("binary", all_commands, binary_append_command,
+		  _("Append target code/data to a raw binary file."),
 		  &binary_append_cmdlist, "append binary ", 
 		  0 /*allow-unknown*/, 
 		  &append_cmdlist);
@@ -786,5 +783,5 @@ OFFSET will be added to the base address of the file (default zero).\n\
 If START and END are given, only the file contents within that range\n\
 (file relative) will be restored to target memory."));
   c->completer = filename_completer;
-  /* FIXME: completers for other commands. */
+  /* FIXME: completers for other commands.  */
 }
