@@ -1,5 +1,5 @@
 /* Helper routines for C++ support in GDB.
-   Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010
+   Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    Contributed by MontaVista Software.
@@ -36,14 +36,23 @@ struct objfile;
 struct type;
 struct demangle_component;
 
+/* A string representing the name of the anonymous namespace used in GDB.  */
+
+#define CP_ANONYMOUS_NAMESPACE_STR "(anonymous namespace)"
+
+/* The length of the string representing the anonymous namespace.  */
+
+#define CP_ANONYMOUS_NAMESPACE_LEN 21
+
 /* This struct is designed to store data from using directives.  It
-   says that names from namespace IMPORT_SRC should be visible within namespace
-   IMPORT_DEST.  These form a linked list; NEXT is the next element of the
-   list.  If the imported namespace or declaration has been aliased within the
-   IMPORT_DEST namespace, ALIAS is set to a string representing the alias.
-   Otherwise, ALIAS is NULL.  DECLARATION is the name of the imported
-   declaration, if this import statement represents one.  Otherwise DECLARATION
-   is NULL and this import statement represents a namespace.
+   says that names from namespace IMPORT_SRC should be visible within
+   namespace IMPORT_DEST.  These form a linked list; NEXT is the next
+   element of the list.  If the imported namespace or declaration has
+   been aliased within the IMPORT_DEST namespace, ALIAS is set to a
+   string representing the alias.  Otherwise, ALIAS is NULL.
+   DECLARATION is the name of the imported declaration, if this import
+   statement represents one.  Otherwise DECLARATION is NULL and this
+   import statement represents a namespace.
 
    C++:      using namespace A;
    Fortran:  use A
@@ -66,15 +75,18 @@ struct demangle_component;
    import_dest = local scope of the import statement even such as ""
    alias = "LOCALNS"
    declaration = NULL
-   The namespace will get imported as the import_dest::LOCALNS namespace.
+   The namespace will get imported as the import_dest::LOCALNS
+   namespace.
 
-   C++ cannot express it, it would be something like:  using localname = A::x;
+   C++ cannot express it, it would be something like: using localname
+   = A::x;
    Fortran:  use A, only localname => x
    import_src = "A"
    import_dest = local scope of the import statement even such as ""
    alias = "localname"
    declaration = "x"
-   The declaration will get imported as localname or `import_dest`localname.  */
+   The declaration will get imported as localname or
+   `import_dest`localname.  */
 
 struct using_direct
 {
@@ -86,7 +98,8 @@ struct using_direct
 
   struct using_direct *next;
 
-  /* Used during import search to temporarily mark this node as searched.  */
+  /* Used during import search to temporarily mark this node as
+     searched.  */
   int searched;
 };
 
@@ -158,6 +171,12 @@ extern struct symbol *cp_lookup_symbol_imports (const char *scope,
                                                 const int declaration_only,
                                                 const int search_parents);
 
+extern struct symbol *cp_lookup_symbol_imports_or_template
+     (const char *scope,
+      const char *name,
+      const struct block *block,
+      const domain_enum domain);
+
 extern struct type *cp_lookup_nested_type (struct type *parent_type,
 					   const char *nested_name,
 					   const struct block *block);
