@@ -1,6 +1,6 @@
 /* DWARF 2 Expression Evaluator.
 
-   Copyright (C) 2001, 2002, 2003, 2005, 2007, 2008
+   Copyright (C) 2001, 2002, 2003, 2005, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    Contributed by Daniel Berlin (dan@dberlin.org)
@@ -25,15 +25,14 @@
 #include "gdbtypes.h"
 #include "value.h"
 #include "gdbcore.h"
-#include "elf/dwarf2.h"
+#include "dwarf2.h"
 #include "dwarf2expr.h"
 #include "gdb_assert.h"
 
 /* Local prototypes.  */
 
 static void execute_stack_op (struct dwarf_expr_context *,
-			      gdb_byte *, gdb_byte *);
-static struct type *unsigned_address_type (int);
+			      const gdb_byte *, const gdb_byte *);
 
 /* Create a new context for the expression evaluator.  */
 
@@ -41,10 +40,12 @@ struct dwarf_expr_context *
 new_dwarf_expr_context (void)
 {
   struct dwarf_expr_context *retval;
+
   retval = xcalloc (1, sizeof (struct dwarf_expr_context));
   retval->stack_len = 0;
   retval->stack_allocated = 10;
-  retval->stack = xmalloc (retval->stack_allocated * sizeof (CORE_ADDR));
+  retval->stack = xmalloc (retval->stack_allocated
+			   * sizeof (struct dwarf_stack_value));
   retval->num_pieces = 0;
   retval->pieces = 0;
   retval->max_recursion_depth = 0x100;
@@ -61,6 +62,22 @@ free_dwarf_expr_context (struct dwarf_expr_context *ctx)
   xfree (ctx);
 }
 
+/* Helper for make_cleanup_free_dwarf_expr_context.  */
+
+static void
+free_dwarf_expr_context_cleanup (void *arg)
+{
+  free_dwarf_expr_context (arg);
+}
+
+/* Return a cleanup that calls free_dwarf_expr_context.  */
+
+struct cleanup *
+make_cleanup_free_dwarf_expr_context (struct dwarf_expr_context *ctx)
+{
+  return make_cleanup (free_dwarf_expr_context_cleanup, ctx);
+}
+
 /* Expand the memory allocated to CTX's stack to contain at least
    NEED more elements than are currently used.  */
 
@@ -70,8 +87,9 @@ dwarf_expr_grow_stack (struct dwarf_expr_context *ctx, size_t need)
   if (ctx->stack_len + need > ctx->stack_allocated)
     {
       size_t newlen = ctx->stack_len + need + 10;
+
       ctx->stack = xrealloc (ctx->stack,
-			     newlen * sizeof (CORE_ADDR));
+			     newlen * sizeof (struct dwarf_stack_value));
       ctx->stack_allocated = newlen;
     }
 }
@@ -79,10 +97,20 @@ dwarf_expr_grow_stack (struct dwarf_expr_context *ctx, size_t need)
 /* Push VALUE onto CTX's stack.  */
 
 void
-dwarf_expr_push (struct dwarf_expr_context *ctx, CORE_ADDR value)
+dwarf_expr_push (struct dwarf_expr_context *ctx, ULONGEST value,
+		 int in_stack_memory)
 {
+  struct dwarf_stack_value *v;
+
+  /* We keep all stack elements within the range defined by the
+     DWARF address size.  */
+  if (ctx->addr_size < sizeof (ULONGEST))
+    value &= ((ULONGEST) 1 << (ctx->addr_size * HOST_CHAR_BIT)) - 1;
+
   dwarf_expr_grow_stack (ctx, 1);
-  ctx->stack[ctx->stack_len++] = value;
+  v = &ctx->stack[ctx->stack_len++];
+  v->value = value;
+  v->in_stack_memory = in_stack_memory;
 }
 
 /* Pop the top item off of CTX's stack.  */
@@ -97,44 +125,133 @@ dwarf_expr_pop (struct dwarf_expr_context *ctx)
 
 /* Retrieve the N'th item on CTX's stack.  */
 
-CORE_ADDR
+ULONGEST
 dwarf_expr_fetch (struct dwarf_expr_context *ctx, int n)
 {
   if (ctx->stack_len <= n)
-     error (_("Asked for position %d of stack, stack only has %d elements on it."),
+     error (_("Asked for position %d of stack, "
+	      "stack only has %d elements on it."),
 	    n, ctx->stack_len);
-  return ctx->stack[ctx->stack_len - (1 + n)];
+  return ctx->stack[ctx->stack_len - (1 + n)].value;
 
 }
 
+/* Retrieve the N'th item on CTX's stack, converted to an address.  */
+
+CORE_ADDR
+dwarf_expr_fetch_address (struct dwarf_expr_context *ctx, int n)
+{
+  ULONGEST result = dwarf_expr_fetch (ctx, n);
+
+  /* For most architectures, calling extract_unsigned_integer() alone
+     is sufficient for extracting an address.  However, some
+     architectures (e.g. MIPS) use signed addresses and using
+     extract_unsigned_integer() will not produce a correct
+     result.  Make sure we invoke gdbarch_integer_to_address()
+     for those architectures which require it.  */
+  if (gdbarch_integer_to_address_p (ctx->gdbarch))
+    {
+      enum bfd_endian byte_order = gdbarch_byte_order (ctx->gdbarch);
+      gdb_byte *buf = alloca (ctx->addr_size);
+      struct type *int_type;
+
+      switch (ctx->addr_size)
+	{
+	case 2:
+	  int_type = builtin_type (ctx->gdbarch)->builtin_uint16;
+	  break;
+	case 4:
+	  int_type = builtin_type (ctx->gdbarch)->builtin_uint32;
+	  break;
+	case 8:
+	  int_type = builtin_type (ctx->gdbarch)->builtin_uint64;
+	  break;
+	default:
+	  internal_error (__FILE__, __LINE__,
+			  _("Unsupported address size.\n"));
+	}
+
+      store_unsigned_integer (buf, ctx->addr_size, byte_order, result);
+      return gdbarch_integer_to_address (ctx->gdbarch, int_type, buf);
+    }
+
+  return (CORE_ADDR) result;
+}
+
+/* Retrieve the in_stack_memory flag of the N'th item on CTX's stack.  */
+
+int
+dwarf_expr_fetch_in_stack_memory (struct dwarf_expr_context *ctx, int n)
+{
+  if (ctx->stack_len <= n)
+     error (_("Asked for position %d of stack, "
+	      "stack only has %d elements on it."),
+	    n, ctx->stack_len);
+  return ctx->stack[ctx->stack_len - (1 + n)].in_stack_memory;
+
+}
+
+/* Return true if the expression stack is empty.  */
+
+static int
+dwarf_expr_stack_empty_p (struct dwarf_expr_context *ctx)
+{
+  return ctx->stack_len == 0;
+}
+
 /* Add a new piece to CTX's piece list.  */
 static void
-add_piece (struct dwarf_expr_context *ctx,
-           int in_reg, CORE_ADDR value, ULONGEST size)
+add_piece (struct dwarf_expr_context *ctx, ULONGEST size, ULONGEST offset)
 {
   struct dwarf_expr_piece *p;
 
   ctx->num_pieces++;
 
-  if (ctx->pieces)
-    ctx->pieces = xrealloc (ctx->pieces,
-                            (ctx->num_pieces
-                             * sizeof (struct dwarf_expr_piece)));
-  else
-    ctx->pieces = xmalloc (ctx->num_pieces
-                           * sizeof (struct dwarf_expr_piece));
+  ctx->pieces = xrealloc (ctx->pieces,
+			  (ctx->num_pieces
+			   * sizeof (struct dwarf_expr_piece)));
 
   p = &ctx->pieces[ctx->num_pieces - 1];
-  p->in_reg = in_reg;
-  p->value = value;
+  p->location = ctx->location;
   p->size = size;
+  p->offset = offset;
+
+  if (p->location == DWARF_VALUE_LITERAL)
+    {
+      p->v.literal.data = ctx->data;
+      p->v.literal.length = ctx->len;
+    }
+  else if (dwarf_expr_stack_empty_p (ctx))
+    {
+      p->location = DWARF_VALUE_OPTIMIZED_OUT;
+      /* Also reset the context's location, for our callers.  This is
+	 a somewhat strange approach, but this lets us avoid setting
+	 the location to DWARF_VALUE_MEMORY in all the individual
+	 cases in the evaluator.  */
+      ctx->location = DWARF_VALUE_OPTIMIZED_OUT;
+    }
+  else if (p->location == DWARF_VALUE_MEMORY)
+    {
+      p->v.mem.addr = dwarf_expr_fetch_address (ctx, 0);
+      p->v.mem.in_stack_memory = dwarf_expr_fetch_in_stack_memory (ctx, 0);
+    }
+  else if (p->location == DWARF_VALUE_IMPLICIT_POINTER)
+    {
+      p->v.ptr.die = ctx->len;
+      p->v.ptr.offset = (LONGEST) dwarf_expr_fetch (ctx, 0);
+    }
+  else
+    {
+      p->v.value = dwarf_expr_fetch (ctx, 0);
+    }
 }
 
 /* Evaluate the expression at ADDR (LEN bytes long) using the context
    CTX.  */
 
 void
-dwarf_expr_eval (struct dwarf_expr_context *ctx, gdb_byte *addr, size_t len)
+dwarf_expr_eval (struct dwarf_expr_context *ctx, const gdb_byte *addr,
+		 size_t len)
 {
   int old_recursion_depth = ctx->recursion_depth;
 
@@ -149,8 +266,8 @@ dwarf_expr_eval (struct dwarf_expr_context *ctx, gdb_byte *addr, size_t len)
    by R, and return the new value of BUF.  Verify that it doesn't extend
    past BUF_END.  */
 
-gdb_byte *
-read_uleb128 (gdb_byte *buf, gdb_byte *buf_end, ULONGEST * r)
+const gdb_byte *
+read_uleb128 (const gdb_byte *buf, const gdb_byte *buf_end, ULONGEST * r)
 {
   unsigned shift = 0;
   ULONGEST result = 0;
@@ -162,7 +279,7 @@ read_uleb128 (gdb_byte *buf, gdb_byte *buf_end, ULONGEST * r)
 	error (_("read_uleb128: Corrupted DWARF expression."));
 
       byte = *buf++;
-      result |= (byte & 0x7f) << shift;
+      result |= ((ULONGEST) (byte & 0x7f)) << shift;
       if ((byte & 0x80) == 0)
 	break;
       shift += 7;
@@ -175,8 +292,8 @@ read_uleb128 (gdb_byte *buf, gdb_byte *buf_end, ULONGEST * r)
    by R, and return the new value of BUF.  Verify that it doesn't extend
    past BUF_END.  */
 
-gdb_byte *
-read_sleb128 (gdb_byte *buf, gdb_byte *buf_end, LONGEST * r)
+const gdb_byte *
+read_sleb128 (const gdb_byte *buf, const gdb_byte *buf_end, LONGEST * r)
 {
   unsigned shift = 0;
   LONGEST result = 0;
@@ -188,7 +305,7 @@ read_sleb128 (gdb_byte *buf, gdb_byte *buf_end, LONGEST * r)
 	error (_("read_sleb128: Corrupted DWARF expression."));
 
       byte = *buf++;
-      result |= (byte & 0x7f) << shift;
+      result |= ((ULONGEST) (byte & 0x7f)) << shift;
       shift += 7;
       if ((byte & 0x80) == 0)
 	break;
@@ -199,94 +316,38 @@ read_sleb128 (gdb_byte *buf, gdb_byte *buf_end, LONGEST * r)
   *r = result;
   return buf;
 }
-
-/* Read an address of size ADDR_SIZE from BUF, and verify that it
-   doesn't extend past BUF_END.  */
-
-CORE_ADDR
-dwarf2_read_address (gdb_byte *buf, gdb_byte *buf_end, int addr_size)
-{
-  CORE_ADDR result;
-
-  if (buf_end - buf < addr_size)
-    error (_("dwarf2_read_address: Corrupted DWARF expression."));
-
-  /* For most architectures, calling extract_unsigned_integer() alone
-     is sufficient for extracting an address.  However, some
-     architectures (e.g. MIPS) use signed addresses and using
-     extract_unsigned_integer() will not produce a correct
-     result.  Turning the unsigned integer into a value and then
-     decomposing that value as an address will cause
-     gdbarch_integer_to_address() to be invoked for those
-     architectures which require it.  Thus, using value_as_address()
-     will produce the correct result for both types of architectures.
-
-     One concern regarding the use of values for this purpose is
-     efficiency.  Obviously, these extra calls will take more time to
-     execute and creating a value takes more space, space which will
-     have to be garbage collected at a later time.  If constructing
-     and then decomposing a value for this purpose proves to be too
-     inefficient, then gdbarch_integer_to_address() can be called
-     directly.
-
-     The use of `unsigned_address_type' in the code below refers to
-     the type of buf and has no bearing on the signedness of the
-     address being returned.  */
-
-  result = value_as_address (value_from_longest 
-			      (unsigned_address_type (addr_size),
-			       extract_unsigned_integer (buf, addr_size)));
-  return result;
-}
-
-/* Return the type of an address of size ADDR_SIZE,
-   for unsigned arithmetic.  */
-
-static struct type *
-unsigned_address_type (int addr_size)
-{
-  switch (addr_size)
-    {
-    case 2:
-      return builtin_type_uint16;
-    case 4:
-      return builtin_type_uint32;
-    case 8:
-      return builtin_type_uint64;
-    default:
-      internal_error (__FILE__, __LINE__,
-		      _("Unsupported address size.\n"));
-    }
-}
-
-/* Return the type of an address of size ADDR_SIZE,
-   for signed arithmetic.  */
-
-static struct type *
-signed_address_type (int addr_size)
-{
-  switch (addr_size)
-    {
-    case 2:
-      return builtin_type_int16;
-    case 4:
-      return builtin_type_int32;
-    case 8:
-      return builtin_type_int64;
-    default:
-      internal_error (__FILE__, __LINE__,
-		      _("Unsupported address size.\n"));
-    }
-}
 
+
+/* Check that the current operator is either at the end of an
+   expression, or that it is followed by a composition operator.  */
+
+void
+dwarf_expr_require_composition (const gdb_byte *op_ptr, const gdb_byte *op_end,
+				const char *op_name)
+{
+  /* It seems like DW_OP_GNU_uninit should be handled here.  However,
+     it doesn't seem to make sense for DW_OP_*_value, and it was not
+     checked at the other place that this function is called.  */
+  if (op_ptr != op_end && *op_ptr != DW_OP_piece && *op_ptr != DW_OP_bit_piece)
+    error (_("DWARF-2 expression error: `%s' operations must be "
+	     "used either alone or in conjuction with DW_OP_piece "
+	     "or DW_OP_bit_piece."),
+	   op_name);
+}
+
 /* The engine for the expression evaluator.  Using the context in CTX,
    evaluate the expression between OP_PTR and OP_END.  */
 
 static void
 execute_stack_op (struct dwarf_expr_context *ctx,
-		  gdb_byte *op_ptr, gdb_byte *op_end)
+		  const gdb_byte *op_ptr, const gdb_byte *op_end)
 {
-  ctx->in_reg = 0;
+#define sign_ext(x) ((LONGEST) (((x) ^ sign_bit) - sign_bit))
+  ULONGEST sign_bit = (ctx->addr_size >= sizeof (ULONGEST) ? 0
+		       : ((ULONGEST) 1) << (ctx->addr_size * 8 - 1));
+  enum bfd_endian byte_order = gdbarch_byte_order (ctx->gdbarch);
+
+  ctx->location = DWARF_VALUE_MEMORY;
   ctx->initialized = 1;  /* Default is initialized.  */
 
   if (ctx->recursion_depth > ctx->max_recursion_depth)
@@ -297,7 +358,14 @@ execute_stack_op (struct dwarf_expr_context *ctx,
   while (op_ptr < op_end)
     {
       enum dwarf_location_atom op = *op_ptr++;
-      CORE_ADDR result;
+      ULONGEST result;
+      /* Assume the value is not in stack memory.
+	 Code that knows otherwise sets this to 1.
+	 Some arithmetic on stack addresses can probably be assumed to still
+	 be a stack address, but we skip this complication for now.
+	 This is just an optimization, so it's always ok to punt
+	 and leave this as 0.  */
+      int in_stack_memory = 0;
       ULONGEST uoffset, reg;
       LONGEST offset;
 
@@ -339,40 +407,47 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	  break;
 
 	case DW_OP_addr:
-	  result = dwarf2_read_address (op_ptr, op_end, ctx->addr_size);
+	  result = extract_unsigned_integer (op_ptr,
+					     ctx->addr_size, byte_order);
 	  op_ptr += ctx->addr_size;
+	  /* Some versions of GCC emit DW_OP_addr before
+	     DW_OP_GNU_push_tls_address.  In this case the value is an
+	     index, not an address.  We don't support things like
+	     branching between the address and the TLS op.  */
+	  if (op_ptr >= op_end || *op_ptr != DW_OP_GNU_push_tls_address)
+	    result += ctx->offset;
 	  break;
 
 	case DW_OP_const1u:
-	  result = extract_unsigned_integer (op_ptr, 1);
+	  result = extract_unsigned_integer (op_ptr, 1, byte_order);
 	  op_ptr += 1;
 	  break;
 	case DW_OP_const1s:
-	  result = extract_signed_integer (op_ptr, 1);
+	  result = extract_signed_integer (op_ptr, 1, byte_order);
 	  op_ptr += 1;
 	  break;
 	case DW_OP_const2u:
-	  result = extract_unsigned_integer (op_ptr, 2);
+	  result = extract_unsigned_integer (op_ptr, 2, byte_order);
 	  op_ptr += 2;
 	  break;
 	case DW_OP_const2s:
-	  result = extract_signed_integer (op_ptr, 2);
+	  result = extract_signed_integer (op_ptr, 2, byte_order);
 	  op_ptr += 2;
 	  break;
 	case DW_OP_const4u:
-	  result = extract_unsigned_integer (op_ptr, 4);
+	  result = extract_unsigned_integer (op_ptr, 4, byte_order);
 	  op_ptr += 4;
 	  break;
 	case DW_OP_const4s:
-	  result = extract_signed_integer (op_ptr, 4);
+	  result = extract_signed_integer (op_ptr, 4, byte_order);
 	  op_ptr += 4;
 	  break;
 	case DW_OP_const8u:
-	  result = extract_unsigned_integer (op_ptr, 8);
+	  result = extract_unsigned_integer (op_ptr, 8, byte_order);
 	  op_ptr += 8;
 	  break;
 	case DW_OP_const8s:
-	  result = extract_signed_integer (op_ptr, 8);
+	  result = extract_signed_integer (op_ptr, 8, byte_order);
 	  op_ptr += 8;
 	  break;
 	case DW_OP_constu:
@@ -420,23 +495,63 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	case DW_OP_reg31:
 	  if (op_ptr != op_end 
 	      && *op_ptr != DW_OP_piece
+	      && *op_ptr != DW_OP_bit_piece
 	      && *op_ptr != DW_OP_GNU_uninit)
 	    error (_("DWARF-2 expression error: DW_OP_reg operations must be "
-		   "used either alone or in conjuction with DW_OP_piece."));
+		     "used either alone or in conjuction with DW_OP_piece "
+		     "or DW_OP_bit_piece."));
 
 	  result = op - DW_OP_reg0;
-	  ctx->in_reg = 1;
-
+	  ctx->location = DWARF_VALUE_REGISTER;
 	  break;
 
 	case DW_OP_regx:
 	  op_ptr = read_uleb128 (op_ptr, op_end, &reg);
-	  if (op_ptr != op_end && *op_ptr != DW_OP_piece)
-	    error (_("DWARF-2 expression error: DW_OP_reg operations must be "
-		   "used either alone or in conjuction with DW_OP_piece."));
+	  dwarf_expr_require_composition (op_ptr, op_end, "DW_OP_regx");
 
 	  result = reg;
-	  ctx->in_reg = 1;
+	  ctx->location = DWARF_VALUE_REGISTER;
+	  break;
+
+	case DW_OP_implicit_value:
+	  {
+	    ULONGEST len;
+
+	    op_ptr = read_uleb128 (op_ptr, op_end, &len);
+	    if (op_ptr + len > op_end)
+	      error (_("DW_OP_implicit_value: too few bytes available."));
+	    ctx->len = len;
+	    ctx->data = op_ptr;
+	    ctx->location = DWARF_VALUE_LITERAL;
+	    op_ptr += len;
+	    dwarf_expr_require_composition (op_ptr, op_end,
+					    "DW_OP_implicit_value");
+	  }
+	  goto no_push;
+
+	case DW_OP_stack_value:
+	  ctx->location = DWARF_VALUE_STACK;
+	  dwarf_expr_require_composition (op_ptr, op_end, "DW_OP_stack_value");
+	  goto no_push;
+
+	case DW_OP_GNU_implicit_pointer:
+	  {
+	    ULONGEST die;
+	    LONGEST len;
+
+	    /* The referred-to DIE.  */
+	    ctx->len = extract_unsigned_integer (op_ptr, ctx->addr_size,
+						 byte_order);
+	    op_ptr += ctx->addr_size;
+
+	    /* The byte offset into the data.  */
+	    op_ptr = read_sleb128 (op_ptr, op_end, &len);
+	    result = (ULONGEST) len;
+
+	    ctx->location = DWARF_VALUE_IMPLICIT_POINTER;
+	    dwarf_expr_require_composition (op_ptr, op_end,
+					    "DW_OP_GNU_implicit_pointer");
+	  }
 	  break;
 
 	case DW_OP_breg0:
@@ -487,7 +602,7 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	  break;
 	case DW_OP_fbreg:
 	  {
-	    gdb_byte *datastart;
+	    const gdb_byte *datastart;
 	    size_t datalen;
 	    unsigned int before_stack_len;
 
@@ -502,16 +617,23 @@ execute_stack_op (struct dwarf_expr_context *ctx,
                specific this_base method.  */
 	    (ctx->get_frame_base) (ctx->baton, &datastart, &datalen);
 	    dwarf_expr_eval (ctx, datastart, datalen);
-	    result = dwarf_expr_fetch (ctx, 0);
-	    if (ctx->in_reg)
-	      result = (ctx->read_reg) (ctx->baton, result);
+	    if (ctx->location == DWARF_VALUE_MEMORY)
+	      result = dwarf_expr_fetch_address (ctx, 0);
+	    else if (ctx->location == DWARF_VALUE_REGISTER)
+	      result = (ctx->read_reg) (ctx->baton, dwarf_expr_fetch (ctx, 0));
+	    else
+	      error (_("Not implemented: computing frame "
+		       "base using explicit value operator"));
 	    result = result + offset;
+	    in_stack_memory = 1;
 	    ctx->stack_len = before_stack_len;
-	    ctx->in_reg = 0;
+	    ctx->location = DWARF_VALUE_MEMORY;
 	  }
 	  break;
+
 	case DW_OP_dup:
 	  result = dwarf_expr_fetch (ctx, 0);
+	  in_stack_memory = dwarf_expr_fetch_in_stack_memory (ctx, 0);
 	  break;
 
 	case DW_OP_drop:
@@ -521,18 +643,36 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	case DW_OP_pick:
 	  offset = *op_ptr++;
 	  result = dwarf_expr_fetch (ctx, offset);
+	  in_stack_memory = dwarf_expr_fetch_in_stack_memory (ctx, offset);
 	  break;
+	  
+	case DW_OP_swap:
+	  {
+	    struct dwarf_stack_value t1, t2;
+
+	    if (ctx->stack_len < 2)
+	       error (_("Not enough elements for "
+			"DW_OP_swap.  Need 2, have %d."),
+		      ctx->stack_len);
+	    t1 = ctx->stack[ctx->stack_len - 1];
+	    t2 = ctx->stack[ctx->stack_len - 2];
+	    ctx->stack[ctx->stack_len - 1] = t2;
+	    ctx->stack[ctx->stack_len - 2] = t1;
+	    goto no_push;
+	  }
 
 	case DW_OP_over:
 	  result = dwarf_expr_fetch (ctx, 1);
+	  in_stack_memory = dwarf_expr_fetch_in_stack_memory (ctx, 1);
 	  break;
 
 	case DW_OP_rot:
 	  {
-	    CORE_ADDR t1, t2, t3;
+	    struct dwarf_stack_value t1, t2, t3;
 
 	    if (ctx->stack_len < 3)
-	       error (_("Not enough elements for DW_OP_rot. Need 3, have %d."),
+	       error (_("Not enough elements for "
+			"DW_OP_rot.  Need 3, have %d."),
 		      ctx->stack_len);
 	    t1 = ctx->stack[ctx->stack_len - 1];
 	    t2 = ctx->stack[ctx->stack_len - 2];
@@ -545,6 +685,17 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 
 	case DW_OP_deref:
 	case DW_OP_deref_size:
+	  {
+	    int addr_size = (op == DW_OP_deref ? ctx->addr_size : *op_ptr++);
+	    gdb_byte *buf = alloca (addr_size);
+	    CORE_ADDR addr = dwarf_expr_fetch_address (ctx, 0);
+	    dwarf_expr_pop (ctx);
+
+	    (ctx->read_mem) (ctx->baton, buf, addr, addr_size);
+	    result = extract_unsigned_integer (buf, addr_size, byte_order);
+	    break;
+	  }
+
 	case DW_OP_abs:
 	case DW_OP_neg:
 	case DW_OP_not:
@@ -555,27 +706,8 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 
 	  switch (op)
 	    {
-	    case DW_OP_deref:
-	      {
-		gdb_byte *buf = alloca (ctx->addr_size);
-		(ctx->read_mem) (ctx->baton, buf, result, ctx->addr_size);
-		result = dwarf2_read_address (buf, buf + ctx->addr_size,
-					      ctx->addr_size);
-	      }
-	      break;
-
-	    case DW_OP_deref_size:
-	      {
-		int addr_size = *op_ptr++;
-		gdb_byte *buf = alloca (addr_size);
-		(ctx->read_mem) (ctx->baton, buf, result, addr_size);
-		result = dwarf2_read_address (buf, buf + addr_size,
-					      addr_size);
-	      }
-	      break;
-
 	    case DW_OP_abs:
-	      if ((signed int) result < 0)
+	      if (sign_ext (result) < 0)
 		result = -result;
 	      break;
 	    case DW_OP_neg:
@@ -609,11 +741,8 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	case DW_OP_gt:
 	case DW_OP_ne:
 	  {
-	    /* Binary operations.  Use the value engine to do computations in
-	       the right width.  */
-	    CORE_ADDR first, second;
-	    enum exp_opcode binop;
-	    struct value *val1, *val2;
+	    /* Binary operations.  */
+	    ULONGEST first, second;
 
 	    second = dwarf_expr_fetch (ctx, 0);
 	    dwarf_expr_pop (ctx);
@@ -621,78 +750,79 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	    first = dwarf_expr_fetch (ctx, 0);
 	    dwarf_expr_pop (ctx);
 
-	    val1 = value_from_longest
-		     (unsigned_address_type (ctx->addr_size), first);
-	    val2 = value_from_longest
-		     (unsigned_address_type (ctx->addr_size), second);
-
 	    switch (op)
 	      {
 	      case DW_OP_and:
-		binop = BINOP_BITWISE_AND;
+		result = first & second;
 		break;
 	      case DW_OP_div:
-		binop = BINOP_DIV;
+		if (!second)
+		  error (_("Division by zero"));
+		result = sign_ext (first) / sign_ext (second);
                 break;
 	      case DW_OP_minus:
-		binop = BINOP_SUB;
+		result = first - second;
 		break;
 	      case DW_OP_mod:
-		binop = BINOP_MOD;
+		if (!second)
+		  error (_("Division by zero"));
+		result = first % second;
 		break;
 	      case DW_OP_mul:
-		binop = BINOP_MUL;
+		result = first * second;
 		break;
 	      case DW_OP_or:
-		binop = BINOP_BITWISE_IOR;
+		result = first | second;
 		break;
 	      case DW_OP_plus:
-		binop = BINOP_ADD;
+		result = first + second;
 		break;
 	      case DW_OP_shl:
-		binop = BINOP_LSH;
+		result = first << second;
 		break;
 	      case DW_OP_shr:
-		binop = BINOP_RSH;
+		result = first >> second;
                 break;
 	      case DW_OP_shra:
-		binop = BINOP_RSH;
-		val1 = value_from_longest
-			 (signed_address_type (ctx->addr_size), first);
+		result = sign_ext (first) >> second;
 		break;
 	      case DW_OP_xor:
-		binop = BINOP_BITWISE_XOR;
+		result = first ^ second;
 		break;
 	      case DW_OP_le:
-		binop = BINOP_LEQ;
+		result = sign_ext (first) <= sign_ext (second);
 		break;
 	      case DW_OP_ge:
-		binop = BINOP_GEQ;
+		result = sign_ext (first) >= sign_ext (second);
 		break;
 	      case DW_OP_eq:
-		binop = BINOP_EQUAL;
+		result = sign_ext (first) == sign_ext (second);
 		break;
 	      case DW_OP_lt:
-		binop = BINOP_LESS;
+		result = sign_ext (first) < sign_ext (second);
 		break;
 	      case DW_OP_gt:
-		binop = BINOP_GTR;
+		result = sign_ext (first) > sign_ext (second);
 		break;
 	      case DW_OP_ne:
-		binop = BINOP_NOTEQUAL;
+		result = sign_ext (first) != sign_ext (second);
 		break;
 	      default:
 		internal_error (__FILE__, __LINE__,
 				_("Can't be reached."));
 	      }
-	    result = value_as_long (value_binop (val1, val2, binop));
 	  }
 	  break;
 
+	case DW_OP_call_frame_cfa:
+	  result = (ctx->get_frame_cfa) (ctx->baton);
+	  in_stack_memory = 1;
+	  break;
+
 	case DW_OP_GNU_push_tls_address:
 	  /* Variable is at a constant offset in the thread-local
 	  storage block into the objfile for the current thread and
-	  the dynamic linker module containing this expression. Here
+	  the dynamic linker module containing this expression.  Here
 	  we return returns the offset from that base.  The top of the
 	  stack has the offset from the beginning of the thread
 	  control block at which the variable is located.  Nothing
@@ -704,13 +834,13 @@ execute_stack_op (struct dwarf_expr_context *ctx,
 	  break;
 
 	case DW_OP_skip:
-	  offset = extract_signed_integer (op_ptr, 2);
+	  offset = extract_signed_integer (op_ptr, 2, byte_order);
 	  op_ptr += 2;
 	  op_ptr += offset;
 	  goto no_push;
 
 	case DW_OP_bra:
-	  offset = extract_signed_integer (op_ptr, 2);
+	  offset = extract_signed_integer (op_ptr, 2, byte_order);
 	  op_ptr += 2;
 	  if (dwarf_expr_fetch (ctx, 0) != 0)
 	    op_ptr += offset;
@@ -723,36 +853,83 @@ execute_stack_op (struct dwarf_expr_context *ctx,
         case DW_OP_piece:
           {
             ULONGEST size;
-            CORE_ADDR addr_or_regnum;
 
             /* Record the piece.  */
             op_ptr = read_uleb128 (op_ptr, op_end, &size);
-            addr_or_regnum = dwarf_expr_fetch (ctx, 0);
-            add_piece (ctx, ctx->in_reg, addr_or_regnum, size);
+	    add_piece (ctx, 8 * size, 0);
 
-            /* Pop off the address/regnum, and clear the in_reg flag.  */
-            dwarf_expr_pop (ctx);
-            ctx->in_reg = 0;
+            /* Pop off the address/regnum, and reset the location
+	       type.  */
+	    if (ctx->location != DWARF_VALUE_LITERAL
+		&& ctx->location != DWARF_VALUE_OPTIMIZED_OUT)
+	      dwarf_expr_pop (ctx);
+            ctx->location = DWARF_VALUE_MEMORY;
           }
           goto no_push;
 
+	case DW_OP_bit_piece:
+	  {
+	    ULONGEST size, offset;
+
+            /* Record the piece.  */
+	    op_ptr = read_uleb128 (op_ptr, op_end, &size);
+	    op_ptr = read_uleb128 (op_ptr, op_end, &offset);
+	    add_piece (ctx, size, offset);
+
+            /* Pop off the address/regnum, and reset the location
+	       type.  */
+	    if (ctx->location != DWARF_VALUE_LITERAL
+		&& ctx->location != DWARF_VALUE_OPTIMIZED_OUT)
+	      dwarf_expr_pop (ctx);
+            ctx->location = DWARF_VALUE_MEMORY;
+	  }
+	  goto no_push;
+
 	case DW_OP_GNU_uninit:
 	  if (op_ptr != op_end)
-	    error (_("DWARF-2 expression error: DW_OP_GNU_unint must always "
+	    error (_("DWARF-2 expression error: DW_OP_GNU_uninit must always "
 		   "be the very last op."));
 
 	  ctx->initialized = 0;
 	  goto no_push;
 
+	case DW_OP_call2:
+	  result = extract_unsigned_integer (op_ptr, 2, byte_order);
+	  op_ptr += 2;
+	  ctx->dwarf_call (ctx, result);
+	  goto no_push;
+
+	case DW_OP_call4:
+	  result = extract_unsigned_integer (op_ptr, 4, byte_order);
+	  op_ptr += 4;
+	  ctx->dwarf_call (ctx, result);
+	  goto no_push;
+	
+	case DW_OP_GNU_entry_value:
+	  /* This operation is not yet supported by GDB.  */
+	  ctx->location = DWARF_VALUE_OPTIMIZED_OUT;
+	  ctx->stack_len = 0;
+	  ctx->num_pieces = 0;
+	  goto abort_expression;
+
 	default:
 	  error (_("Unhandled dwarf expression opcode 0x%x"), op);
 	}
 
       /* Most things push a result value.  */
-      dwarf_expr_push (ctx, result);
-    no_push:;
+      dwarf_expr_push (ctx, result, in_stack_memory);
+    no_push:
+      ;
     }
 
+  /* To simplify our main caller, if the result is an implicit
+     pointer, then make a pieced value.  This is ok because we can't
+     have implicit pointers in contexts where pieces are invalid.  */
+  if (ctx->location == DWARF_VALUE_IMPLICIT_POINTER)
+    add_piece (ctx, 8 * ctx->addr_size, 0);
+
+abort_expression:
   ctx->recursion_depth--;
   gdb_assert (ctx->recursion_depth >= 0);
+#undef sign_ext
 }
