@@ -1,6 +1,6 @@
 /* Event loop machinery for GDB, the GNU debugger.
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2006, 2007, 2008, 2009
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2006, 2007, 2008, 2009, 2010,
+   2011 Free Software Foundation, Inc.
    Written by Elena Zannoni <ezannoni@cygnus.com> of Cygnus Solutions.
 
    This file is part of GDB.
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "event-loop.h"
@@ -38,6 +38,13 @@
 #include "gdb_assert.h"
 #include "gdb_select.h"
 
+/* Tell create_file_handler what events we are interested in.
+   This is used by the select version of the event loop.  */
+
+#define GDB_READABLE	(1<<1)
+#define GDB_WRITABLE	(1<<2)
+#define GDB_EXCEPTION	(1<<3)
+
 /* Data point to pass to the event handler.  */
 typedef union event_data
 {
@@ -49,12 +56,12 @@ typedef struct gdb_event gdb_event;
 typedef void (event_handler_func) (event_data);
 
 /* Event for the GDB event system.  Events are queued by calling
-   async_queue_event and serviced later on by gdb_do_one_event. An
+   async_queue_event and serviced later on by gdb_do_one_event.  An
    event can be, for instance, a file descriptor becoming ready to be
    read.  Servicing an event simply means that the procedure PROC will
    be called.  We have 2 queues, one for file handlers that we listen
    to in the event loop, and one for the file handlers+events that are
-   ready. The procedure PROC associated with each event is dependant
+   ready.  The procedure PROC associated with each event is dependant
    of the event source.  In the case of monitored file descriptors, it
    is always the same (handle_file_event).  Its duty is to invoke the
    handler associated with the file descriptor whose state change
@@ -75,36 +82,39 @@ struct gdb_event
   };
 
 /* Information about each file descriptor we register with the event
-   loop. */
+   loop.  */
 
 typedef struct file_handler
   {
-    int fd;			/* File descriptor. */
-    int mask;			/* Events we want to monitor: POLLIN, etc. */
+    int fd;			/* File descriptor.  */
+    int mask;			/* Events we want to monitor: POLLIN, etc.  */
     int ready_mask;		/* Events that have been seen since
-				   the last time. */
-    handler_func *proc;		/* Procedure to call when fd is ready. */
-    gdb_client_data client_data;	/* Argument to pass to proc. */
-    int error;			/* Was an error detected on this fd? */
-    struct file_handler *next_file;	/* Next registered file descriptor. */
+				   the last time.  */
+    handler_func *proc;		/* Procedure to call when fd is ready.  */
+    gdb_client_data client_data;	/* Argument to pass to proc.  */
+    int error;			/* Was an error detected on this fd?  */
+    struct file_handler *next_file;	/* Next registered file descriptor.  */
   }
 file_handler;
 
-/* PROC is a function to be invoked when the READY flag is set. This
+/* PROC is a function to be invoked when the READY flag is set.  This
    happens when there has been a signal and the corresponding signal
-   handler has 'triggered' this async_signal_handler for
-   execution. The actual work to be done in response to a signal will
-   be carried out by PROC at a later time, within process_event. This
-   provides a deferred execution of signal handlers.
+   handler has 'triggered' this async_signal_handler for execution.
+   The actual work to be done in response to a signal will be carried
+   out by PROC at a later time, within process_event.  This provides a
+   deferred execution of signal handlers.
+
    Async_init_signals takes care of setting up such an
-   async_signal_handler for each interesting signal. */
+   async_signal_handler for each interesting signal.  */
+
 typedef struct async_signal_handler
   {
-    int ready;			/* If ready, call this handler from the main event loop, 
-				   using invoke_async_handler. */
-    struct async_signal_handler *next_handler;	/* Ptr to next handler */
-    sig_handler_func *proc;	/* Function to call to do the work */
-    gdb_client_data client_data;	/* Argument to async_handler_func */
+    int ready;			    /* If ready, call this handler
+				       from the main event loop, using
+				       invoke_async_handler.  */
+    struct async_signal_handler *next_handler;	/* Ptr to next handler.  */
+    sig_handler_func *proc;	    /* Function to call to do the work.  */
+    gdb_client_data client_data;    /* Argument to async_handler_func.  */
   }
 async_signal_handler;
 
@@ -133,7 +143,7 @@ async_event_handler;
 
 
 /* Event queue:  
-   - the first event in the queue is the head of the queue. 
+   - the first event in the queue is the head of the queue.
    It will be the next to be serviced.
    - the last event in the queue 
 
@@ -143,25 +153,25 @@ async_event_handler;
    the queue will be processed in a last in first out fashion, while
    those inserted at the tail of the queue will be processed in a first
    in first out manner.  All the fields are NULL if the queue is
-   empty. */
+   empty.  */
 
 static struct
   {
-    gdb_event *first_event;	/* First pending event */
-    gdb_event *last_event;	/* Last pending event */
+    gdb_event *first_event;	/* First pending event.  */
+    gdb_event *last_event;	/* Last pending event.  */
   }
 event_queue;
 
 /* Gdb_notifier is just a list of file descriptors gdb is interested in.
    These are the input file descriptor, and the target file
-   descriptor. We have two flavors of the notifier, one for platforms
+   descriptor.  We have two flavors of the notifier, one for platforms
    that have the POLL function, the other for those that don't, and
-   only support SELECT. Each of the elements in the gdb_notifier list is
+   only support SELECT.  Each of the elements in the gdb_notifier list is
    basically a description of what kind of events gdb is interested
-   in, for each fd. */
+   in, for each fd.  */
 
 /* As of 1999-04-30 only the input file descriptor is registered with the
-   event loop. */
+   event loop.  */
 
 /* Do we use poll or select ? */
 #ifdef HAVE_POLL
@@ -179,80 +189,79 @@ static unsigned char use_poll = USE_POLL;
 
 static struct
   {
-    /* Ptr to head of file handler list. */
+    /* Ptr to head of file handler list.  */
     file_handler *first_file_handler;
 
 #ifdef HAVE_POLL
-    /* Ptr to array of pollfd structures. */
+    /* Ptr to array of pollfd structures.  */
     struct pollfd *poll_fds;
 
-    /* Timeout in milliseconds for calls to poll(). */
+    /* Timeout in milliseconds for calls to poll().  */
     int poll_timeout;
 #endif
 
     /* Masks to be used in the next call to select.
-       Bits are set in response to calls to create_file_handler. */
+       Bits are set in response to calls to create_file_handler.  */
     fd_set check_masks[3];
 
-    /* What file descriptors were found ready by select. */
+    /* What file descriptors were found ready by select.  */
     fd_set ready_masks[3];
 
-    /* Number of file descriptors to monitor. (for poll) */
-    /* Number of valid bits (highest fd value + 1). (for select) */
+    /* Number of file descriptors to monitor (for poll).  */
+    /* Number of valid bits (highest fd value + 1) (for select).  */
     int num_fds;
 
-    /* Time structure for calls to select(). */
+    /* Time structure for calls to select().  */
     struct timeval select_timeout;
 
-    /* Flag to tell whether the timeout should be used. */
+    /* Flag to tell whether the timeout should be used.  */
     int timeout_valid;
   }
 gdb_notifier;
 
-/* Structure associated with a timer. PROC will be executed at the
-   first occasion after WHEN. */
+/* Structure associated with a timer.  PROC will be executed at the
+   first occasion after WHEN.  */
 struct gdb_timer
   {
     struct timeval when;
     int timer_id;
     struct gdb_timer *next;
-    timer_handler_func *proc;	/* Function to call to do the work */
-    gdb_client_data client_data;	/* Argument to async_handler_func */
-  }
-gdb_timer;
+    timer_handler_func *proc;	    /* Function to call to do the work.  */
+    gdb_client_data client_data;    /* Argument to async_handler_func.  */
+  };
 
-/* List of currently active timers. It is sorted in order of
-   increasing timers. */
+/* List of currently active timers.  It is sorted in order of
+   increasing timers.  */
 static struct
   {
-    /* Pointer to first in timer list. */
+    /* Pointer to first in timer list.  */
     struct gdb_timer *first_timer;
 
-    /* Id of the last timer created. */
+    /* Id of the last timer created.  */
     int num_timers;
   }
 timer_list;
 
 /* All the async_signal_handlers gdb is interested in are kept onto
-   this list. */
+   this list.  */
 static struct
   {
-    /* Pointer to first in handler list. */
+    /* Pointer to first in handler list.  */
     async_signal_handler *first_handler;
 
-    /* Pointer to last in handler list. */
+    /* Pointer to last in handler list.  */
     async_signal_handler *last_handler;
   }
 sighandler_list;
 
 /* All the async_event_handlers gdb is interested in are kept onto
-   this list. */
+   this list.  */
 static struct
   {
-    /* Pointer to first in handler list. */
+    /* Pointer to first in handler list.  */
     async_event_handler *first_handler;
 
-    /* Pointer to last in handler list. */
+    /* Pointer to last in handler list.  */
     async_event_handler *last_handler;
   }
 async_event_handler_list;
@@ -262,7 +271,6 @@ static void create_file_handler (int fd, int mask, handler_func *proc,
 				 gdb_client_data client_data);
 static void handle_file_event (event_data data);
 static void check_async_event_handlers (void);
-static void check_async_signal_handlers (void);
 static int gdb_wait_for_event (int);
 static void poll_timers (void);
 
@@ -271,18 +279,18 @@ static void poll_timers (void);
    the specified position.
    POSITION can be head or tail, with values TAIL, HEAD.
    EVENT_PTR points to the event to be inserted into the queue.
-   The caller must allocate memory for the event. It is freed
+   The caller must allocate memory for the event.  It is freed
    after the event has ben handled.
    Events in the queue will be processed head to tail, therefore,
    events inserted at the head of the queue will be processed
-   as last in first out. Event appended at the tail of the queue
-   will be processed first in first out. */
+   as last in first out.  Event appended at the tail of the queue
+   will be processed first in first out.  */
 static void
 async_queue_event (gdb_event * event_ptr, queue_position position)
 {
   if (position == TAIL)
     {
-      /* The event will become the new last_event. */
+      /* The event will become the new last_event.  */
 
       event_ptr->next_event = NULL;
       if (event_queue.first_event == NULL)
@@ -293,7 +301,7 @@ async_queue_event (gdb_event * event_ptr, queue_position position)
     }
   else if (position == HEAD)
     {
-      /* The event becomes the new first_event. */
+      /* The event becomes the new first_event.  */
 
       event_ptr->next_event = event_queue.first_event;
       if (event_queue.first_event == NULL)
@@ -319,9 +327,9 @@ create_event (event_handler_func proc, event_data data)
 }
 
 /* Create a file event, to be enqueued in the event queue for
-   processing. The procedure associated to this event is always
+   processing.  The procedure associated to this event is always
    handle_file_event, which will in turn invoke the one that was
-   associated to FD when it was registered with the event loop. */
+   associated to FD when it was registered with the event loop.  */
 static gdb_event *
 create_file_event (int fd)
 {
@@ -339,7 +347,7 @@ create_file_event (int fd)
    0 is returned.
    Scan the queue from head to tail, processing therefore the high
    priority events first, by invoking the associated event handler
-   procedure. */
+   procedure.  */
 static int
 process_event (void)
 {
@@ -348,19 +356,19 @@ process_event (void)
   event_data data;
 
   /* First let's see if there are any asynchronous event handlers that
-     are ready. These would be the result of invoking any of the
-     signal handlers. */
+     are ready.  These would be the result of invoking any of the
+     signal handlers.  */
 
   if (invoke_async_signal_handlers ())
     return 1;
 
   /* Look in the event queue to find an event that is ready
-     to be processed. */
+     to be processed.  */
 
   for (event_ptr = event_queue.first_event; event_ptr != NULL;
        event_ptr = event_ptr->next_event)
     {
-      /* Call the handler for the event. */
+      /* Call the handler for the event.  */
 
       proc = event_ptr->proc;
       data = event_ptr->data;
@@ -368,9 +376,9 @@ process_event (void)
       /* Let's get rid of the event from the event queue.  We need to
          do this now because while processing the event, the proc
          function could end up calling 'error' and therefore jump out
-         to the caller of this function, gdb_do_one_event. In that
+         to the caller of this function, gdb_do_one_event.  In that
          case, we would have on the event queue an event wich has been
-         processed, but not deleted. */
+         processed, but not deleted.  */
 
       if (event_queue.first_event == event_ptr)
 	{
@@ -390,12 +398,12 @@ process_event (void)
 	}
       xfree (event_ptr);
 
-      /* Now call the procedure associated with the event. */
+      /* Now call the procedure associated with the event.  */
       (*proc) (data);
       return 1;
     }
 
-  /* this is the case if there are no event on the event queue. */
+  /* This is the case if there are no event on the event queue.  */
   return 0;
 }
 
@@ -403,7 +411,7 @@ process_event (void)
    wait for something to happen (via gdb_wait_for_event), then process
    it.  Returns >0 if something was done otherwise returns <0 (this
    can happen if there are no event sources to wait for).  If an error
-   occurs catch_errors() which calls this function returns zero. */
+   occurs catch_errors() which calls this function returns zero.  */
 
 int
 gdb_do_one_event (void *data)
@@ -424,7 +432,7 @@ gdb_do_one_event (void *data)
 	{
 	case 0:
 	  /* Are any timers that are ready? If so, put an event on the
-	     queue. */
+	     queue.  */
 	  poll_timers ();
 	  break;
 	case 1:
@@ -464,18 +472,18 @@ gdb_do_one_event (void *data)
   return 1;
 }
 
-/* Start up the event loop. This is the entry point to the event loop
-   from the command loop. */
+/* Start up the event loop.  This is the entry point to the event loop
+   from the command loop.  */
 
 void
 start_event_loop (void)
 {
-  /* Loop until there is nothing to do. This is the entry point to the
-     event loop engine. gdb_do_one_event, called via catch_errors()
+  /* Loop until there is nothing to do.  This is the entry point to the
+     event loop engine.  gdb_do_one_event, called via catch_errors()
      will process one event for each invocation.  It blocks waits for
      an event and then processes it.  >0 when an event is processed, 0
      when catch_errors() caught an error and <0 when there are no
-     longer any event sources registered. */
+     longer any event sources registered.  */
   while (1)
     {
       int gdb_result;
@@ -486,7 +494,7 @@ start_event_loop (void)
 
       /* If we long-jumped out of do_one_event, we probably
          didn't get around to resetting the prompt, which leaves
-         readline in a messed-up state.  Reset it here. */
+         readline in a messed-up state.  Reset it here.  */
 
       if (gdb_result == 0)
 	{
@@ -496,7 +504,7 @@ start_event_loop (void)
 	  async_enable_stdin ();
 	  /* FIXME: this should really be a call to a hook that is
 	     interface specific, because interfaces can display the
-	     prompt in their own way. */
+	     prompt in their own way.  */
 	  display_gdb_prompt (0);
 	  /* This call looks bizarre, but it is required.  If the user
 	     entered a command that caused an error,
@@ -507,19 +515,19 @@ start_event_loop (void)
 	  if (after_char_processing_hook)
 	    (*after_char_processing_hook) ();
 	  /* Maybe better to set a flag to be checked somewhere as to
-	     whether display the prompt or not. */
+	     whether display the prompt or not.  */
 	}
     }
 
-  /* We are done with the event loop. There are no more event sources
-     to listen to.  So we exit GDB. */
+  /* We are done with the event loop.  There are no more event sources
+     to listen to.  So we exit GDB.  */
   return;
 }
 
 
 /* Wrapper function for create_file_handler, so that the caller
    doesn't have to know implementation details about the use of poll
-   vs. select. */
+   vs. select.  */
 void
 add_file_handler (int fd, handler_func * proc, gdb_client_data client_data)
 {
@@ -530,11 +538,11 @@ add_file_handler (int fd, handler_func * proc, gdb_client_data client_data)
   if (use_poll)
     {
 #ifdef HAVE_POLL
-      /* Check to see if poll () is usable. If not, we'll switch to
-         use select. This can happen on systems like
+      /* Check to see if poll () is usable.  If not, we'll switch to
+         use select.  This can happen on systems like
          m68k-motorola-sys, `poll' cannot be used to wait for `stdin'.
          On m68k-motorola-sysv, tty's are not stream-based and not
-         `poll'able. */
+         `poll'able.  */
       fds.fd = fd;
       fds.events = POLLIN;
       if (poll (&fds, 1, 0) == 1 && (fds.revents & POLLNVAL))
@@ -554,26 +562,32 @@ add_file_handler (int fd, handler_func * proc, gdb_client_data client_data)
 #endif
     }
   else
-    create_file_handler (fd, GDB_READABLE | GDB_EXCEPTION, proc, client_data);
+    create_file_handler (fd, GDB_READABLE | GDB_EXCEPTION, 
+			 proc, client_data);
 }
 
 /* Add a file handler/descriptor to the list of descriptors we are
-   interested in.  
-   FD is the file descriptor for the file/stream to be listened to.  
-   For the poll case, MASK is a combination (OR) of
-   POLLIN, POLLRDNORM, POLLRDBAND, POLLPRI, POLLOUT, POLLWRNORM,
-   POLLWRBAND: these are the events we are interested in. If any of them 
-   occurs, proc should be called.
-   For the select case, MASK is a combination of READABLE, WRITABLE, EXCEPTION.
-   PROC is the procedure that will be called when an event occurs for
-   FD.  CLIENT_DATA is the argument to pass to PROC. */
+   interested in.
+
+   FD is the file descriptor for the file/stream to be listened to.
+
+   For the poll case, MASK is a combination (OR) of POLLIN,
+   POLLRDNORM, POLLRDBAND, POLLPRI, POLLOUT, POLLWRNORM, POLLWRBAND:
+   these are the events we are interested in.  If any of them occurs,
+   proc should be called.
+
+   For the select case, MASK is a combination of READABLE, WRITABLE,
+   EXCEPTION.  PROC is the procedure that will be called when an event
+   occurs for FD.  CLIENT_DATA is the argument to pass to PROC.  */
+
 static void
-create_file_handler (int fd, int mask, handler_func * proc, gdb_client_data client_data)
+create_file_handler (int fd, int mask, handler_func * proc, 
+		     gdb_client_data client_data)
 {
   file_handler *file_ptr;
 
-  /* Do we already have a file handler for this file? (We may be
-     changing its associated procedure). */
+  /* Do we already have a file handler for this file?  (We may be
+     changing its associated procedure).  */
   for (file_ptr = gdb_notifier.first_file_handler; file_ptr != NULL;
        file_ptr = file_ptr->next_file)
     {
@@ -581,8 +595,8 @@ create_file_handler (int fd, int mask, handler_func * proc, gdb_client_data clie
 	break;
     }
 
-  /* It is a new file descriptor. Add it to the list. Otherwise, just
-     change the data associated with it. */
+  /* It is a new file descriptor.  Add it to the list.  Otherwise, just
+     change the data associated with it.  */
   if (file_ptr == NULL)
     {
       file_ptr = (file_handler *) xmalloc (sizeof (file_handler));
@@ -639,7 +653,7 @@ create_file_handler (int fd, int mask, handler_func * proc, gdb_client_data clie
 }
 
 /* Remove the file descriptor FD from the list of monitored fd's: 
-   i.e. we don't care anymore about events on the FD. */
+   i.e. we don't care anymore about events on the FD.  */
 void
 delete_file_handler (int fd)
 {
@@ -650,7 +664,7 @@ delete_file_handler (int fd)
   struct pollfd *new_poll_fds;
 #endif
 
-  /* Find the entry for the given file. */
+  /* Find the entry for the given file.  */
 
   for (file_ptr = gdb_notifier.first_file_handler; file_ptr != NULL;
        file_ptr = file_ptr->next_file)
@@ -665,11 +679,11 @@ delete_file_handler (int fd)
   if (use_poll)
     {
 #ifdef HAVE_POLL
-      /* Create a new poll_fds array by copying every fd's information but the
-         one we want to get rid of. */
+      /* Create a new poll_fds array by copying every fd's information
+         but the one we want to get rid of.  */
 
-      new_poll_fds =
-	(struct pollfd *) xmalloc ((gdb_notifier.num_fds - 1) * sizeof (struct pollfd));
+      new_poll_fds = (struct pollfd *) 
+	xmalloc ((gdb_notifier.num_fds - 1) * sizeof (struct pollfd));
 
       for (i = 0, j = 0; i < gdb_notifier.num_fds; i++)
 	{
@@ -677,7 +691,8 @@ delete_file_handler (int fd)
 	    {
 	      (new_poll_fds + j)->fd = (gdb_notifier.poll_fds + i)->fd;
 	      (new_poll_fds + j)->events = (gdb_notifier.poll_fds + i)->events;
-	      (new_poll_fds + j)->revents = (gdb_notifier.poll_fds + i)->revents;
+	      (new_poll_fds + j)->revents
+		= (gdb_notifier.poll_fds + i)->revents;
 	      j++;
 	    }
 	}
@@ -698,7 +713,7 @@ delete_file_handler (int fd)
       if (file_ptr->mask & GDB_EXCEPTION)
 	FD_CLR (fd, &gdb_notifier.check_masks[2]);
 
-      /* Find current max fd. */
+      /* Find current max fd.  */
 
       if ((fd + 1) == gdb_notifier.num_fds)
 	{
@@ -715,11 +730,11 @@ delete_file_handler (int fd)
     }
 
   /* Deactivate the file descriptor, by clearing its mask, 
-     so that it will not fire again. */
+     so that it will not fire again.  */
 
   file_ptr->mask = 0;
 
-  /* Get rid of the file handler in the file handler list. */
+  /* Get rid of the file handler in the file handler list.  */
   if (file_ptr == gdb_notifier.first_file_handler)
     gdb_notifier.first_file_handler = file_ptr->next_file;
   else
@@ -736,7 +751,7 @@ delete_file_handler (int fd)
 /* Handle the given event by calling the procedure associated to the
    corresponding file handler.  Called by process_event indirectly,
    through event_ptr->proc.  EVENT_FILE_DESC is file descriptor of the
-   event in the front of the event queue. */
+   event in the front of the event queue.  */
 static void
 handle_file_event (event_data data)
 {
@@ -749,21 +764,21 @@ handle_file_event (event_data data)
   int event_file_desc = data.integer;
 
   /* Search the file handler list to find one that matches the fd in
-     the event. */
+     the event.  */
   for (file_ptr = gdb_notifier.first_file_handler; file_ptr != NULL;
        file_ptr = file_ptr->next_file)
     {
       if (file_ptr->fd == event_file_desc)
 	{
 	  /* With poll, the ready_mask could have any of three events
-	     set to 1: POLLHUP, POLLERR, POLLNVAL. These events cannot
-	     be used in the requested event mask (events), but they
-	     can be returned in the return mask (revents). We need to
-	     check for those event too, and add them to the mask which
-	     will be passed to the handler. */
+	     set to 1: POLLHUP, POLLERR, POLLNVAL.  These events
+	     cannot be used in the requested event mask (events), but
+	     they can be returned in the return mask (revents).  We
+	     need to check for those event too, and add them to the
+	     mask which will be passed to the handler.  */
 
 	  /* See if the desired events (mask) match the received
-	     events (ready_mask). */
+	     events (ready_mask).  */
 
 	  if (use_poll)
 	    {
@@ -775,14 +790,17 @@ handle_file_event (event_data data)
 
 	      if (error_mask_returned != 0)
 		{
-		  /* Work in progress. We may need to tell somebody what
-		     kind of error we had. */
+		  /* Work in progress.  We may need to tell somebody
+		     what kind of error we had.  */
 		  if (error_mask_returned & POLLHUP)
-		    printf_unfiltered (_("Hangup detected on fd %d\n"), file_ptr->fd);
+		    printf_unfiltered (_("Hangup detected on fd %d\n"),
+				       file_ptr->fd);
 		  if (error_mask_returned & POLLERR)
-		    printf_unfiltered (_("Error detected on fd %d\n"), file_ptr->fd);
+		    printf_unfiltered (_("Error detected on fd %d\n"),
+				       file_ptr->fd);
 		  if (error_mask_returned & POLLNVAL)
-		    printf_unfiltered (_("Invalid or non-`poll'able fd %d\n"), file_ptr->fd);
+		    printf_unfiltered (_("Invalid or non-`poll'able fd %d\n"),
+				       file_ptr->fd);
 		  file_ptr->error = 1;
 		}
 	      else
@@ -796,7 +814,8 @@ handle_file_event (event_data data)
 	    {
 	      if (file_ptr->ready_mask & GDB_EXCEPTION)
 		{
-		  printf_unfiltered (_("Exception condition detected on fd %d\n"), file_ptr->fd);
+		  printf_unfiltered (_("Exception condition detected "
+				       "on fd %d\n"), file_ptr->fd);
 		  file_ptr->error = 1;
 		}
 	      else
@@ -804,10 +823,10 @@ handle_file_event (event_data data)
 	      mask = file_ptr->ready_mask & file_ptr->mask;
 	    }
 
-	  /* Clear the received events for next time around. */
+	  /* Clear the received events for next time around.  */
 	  file_ptr->ready_mask = 0;
 
-	  /* If there was a match, then call the handler. */
+	  /* If there was a match, then call the handler.  */
 	  if (mask != 0)
 	    (*file_ptr->proc) (file_ptr->error, file_ptr->client_data);
 	  break;
@@ -818,8 +837,8 @@ handle_file_event (event_data data)
 /* Called by gdb_do_one_event to wait for new events on the monitored
    file descriptors.  Queue file events as they are detected by the
    poll.  If BLOCK and if there are no events, this function will
-   block in the call to poll.  Return -1 if there are no files
-   descriptors to monitor, otherwise return 0. */
+   block in the call to poll.  Return -1 if there are no file
+   descriptors to monitor, otherwise return 0.  */
 static int
 gdb_wait_for_event (int block)
 {
@@ -828,7 +847,7 @@ gdb_wait_for_event (int block)
   int num_found = 0;
   int i;
 
-  /* Make sure all output is done before getting another event. */
+  /* Make sure all output is done before getting another event.  */
   gdb_flush (gdb_stdout);
   gdb_flush (gdb_stderr);
 
@@ -860,8 +879,8 @@ gdb_wait_for_event (int block)
   else
     {
       struct timeval select_timeout;
-
       struct timeval *timeout_p;
+
       if (block)
 	timeout_p = gdb_notifier.timeout_valid
 	  ? &gdb_notifier.select_timeout : NULL;
@@ -880,7 +899,7 @@ gdb_wait_for_event (int block)
 			      &gdb_notifier.ready_masks[2],
 			      timeout_p);
 
-      /* Clear the masks after an error from select. */
+      /* Clear the masks after an error from select.  */
       if (num_found == -1)
 	{
 	  FD_ZERO (&gdb_notifier.ready_masks[0]);
@@ -894,7 +913,7 @@ gdb_wait_for_event (int block)
 	}
     }
 
-  /* Enqueue all detected file events. */
+  /* Enqueue all detected file events.  */
 
   if (use_poll)
     {
@@ -917,7 +936,7 @@ gdb_wait_for_event (int block)
 	  if (file_ptr)
 	    {
 	      /* Enqueue an event only if this is still a new event for
-	         this fd. */
+	         this fd.  */
 	      if (file_ptr->ready_mask == 0)
 		{
 		  file_event_ptr = create_file_event (file_ptr->fd);
@@ -952,7 +971,7 @@ gdb_wait_for_event (int block)
 	    num_found--;
 
 	  /* Enqueue an event only if this is still a new event for
-	     this fd. */
+	     this fd.  */
 
 	  if (file_ptr->ready_mask == 0)
 	    {
@@ -966,14 +985,15 @@ gdb_wait_for_event (int block)
 }
 
 
-/* Create an asynchronous handler, allocating memory for it. 
+/* Create an asynchronous handler, allocating memory for it.
    Return a pointer to the newly created handler.
    This pointer will be used to invoke the handler by 
    invoke_async_signal_handler.
    PROC is the function to call with CLIENT_DATA argument 
-   whenever the handler is invoked. */
+   whenever the handler is invoked.  */
 async_signal_handler *
-create_async_signal_handler (sig_handler_func * proc, gdb_client_data client_data)
+create_async_signal_handler (sig_handler_func * proc,
+			     gdb_client_data client_data)
 {
   async_signal_handler *async_handler_ptr;
 
@@ -1000,10 +1020,10 @@ call_async_signal_handler (struct async_signal_handler *handler)
   (*handler->proc) (handler->client_data);
 }
 
-/* Mark the handler (ASYNC_HANDLER_PTR) as ready. This information will
-   be used when the handlers are invoked, after we have waited for
-   some event.  The caller of this function is the interrupt handler
-   associated with a signal. */
+/* Mark the handler (ASYNC_HANDLER_PTR) as ready.  This information
+   will be used when the handlers are invoked, after we have waited
+   for some event.  The caller of this function is the interrupt
+   handler associated with a signal.  */
 void
 mark_async_signal_handler (async_signal_handler * async_handler_ptr)
 {
@@ -1039,7 +1059,7 @@ invoke_async_signal_handlers (void)
   return any_ready;
 }
 
-/* Delete an asynchronous handler (ASYNC_HANDLER_PTR). 
+/* Delete an asynchronous handler (ASYNC_HANDLER_PTR).
    Free the space allocated for it.  */
 void
 delete_async_signal_handler (async_signal_handler ** async_handler_ptr)
@@ -1057,6 +1077,7 @@ delete_async_signal_handler (async_signal_handler ** async_handler_ptr)
       prev_ptr = sighandler_list.first_handler;
       while (prev_ptr && prev_ptr->next_handler != (*async_handler_ptr))
 	prev_ptr = prev_ptr->next_handler;
+      gdb_assert (prev_ptr);
       prev_ptr->next_handler = (*async_handler_ptr)->next_handler;
       if (sighandler_list.last_handler == (*async_handler_ptr))
 	sighandler_list.last_handler = prev_ptr;
@@ -1155,7 +1176,8 @@ delete_async_event_handler (async_event_handler **async_handler_ptr)
 
   if (async_event_handler_list.first_handler == *async_handler_ptr)
     {
-      async_event_handler_list.first_handler = (*async_handler_ptr)->next_handler;
+      async_event_handler_list.first_handler
+	= (*async_handler_ptr)->next_handler;
       if (async_event_handler_list.first_handler == NULL)
 	async_event_handler_list.last_handler = NULL;
     }
@@ -1164,6 +1186,7 @@ delete_async_event_handler (async_event_handler **async_handler_ptr)
       prev_ptr = async_event_handler_list.first_handler;
       while (prev_ptr && prev_ptr->next_handler != *async_handler_ptr)
 	prev_ptr = prev_ptr->next_handler;
+      gdb_assert (prev_ptr);
       prev_ptr->next_handler = (*async_handler_ptr)->next_handler;
       if (async_event_handler_list.last_handler == (*async_handler_ptr))
 	async_event_handler_list.last_handler = prev_ptr;
@@ -1172,27 +1195,28 @@ delete_async_event_handler (async_event_handler **async_handler_ptr)
   *async_handler_ptr = NULL;
 }
 
-/* Create a timer that will expire in MILLISECONDS from now. When the
-   timer is ready, PROC will be executed. At creation, the timer is
+/* Create a timer that will expire in MILLISECONDS from now.  When the
+   timer is ready, PROC will be executed.  At creation, the timer is
    aded to the timers queue.  This queue is kept sorted in order of
-   increasing timers. Return a handle to the timer struct. */
+   increasing timers.  Return a handle to the timer struct.  */
 int
-create_timer (int milliseconds, timer_handler_func * proc, gdb_client_data client_data)
+create_timer (int milliseconds, timer_handler_func * proc, 
+	      gdb_client_data client_data)
 {
   struct gdb_timer *timer_ptr, *timer_index, *prev_timer;
   struct timeval time_now, delta;
 
-  /* compute seconds */
+  /* Compute seconds.  */
   delta.tv_sec = milliseconds / 1000;
-  /* compute microseconds */
+  /* Compute microseconds.  */
   delta.tv_usec = (milliseconds % 1000) * 1000;
 
   gettimeofday (&time_now, NULL);
 
-  timer_ptr = (struct gdb_timer *) xmalloc (sizeof (gdb_timer));
+  timer_ptr = (struct gdb_timer *) xmalloc (sizeof (*timer_ptr));
   timer_ptr->when.tv_sec = time_now.tv_sec + delta.tv_sec;
   timer_ptr->when.tv_usec = time_now.tv_usec + delta.tv_usec;
-  /* carry? */
+  /* Carry?  */
   if (timer_ptr->when.tv_usec >= 1000000)
     {
       timer_ptr->when.tv_sec += 1;
@@ -1204,17 +1228,17 @@ create_timer (int milliseconds, timer_handler_func * proc, gdb_client_data clien
   timer_ptr->timer_id = timer_list.num_timers;
 
   /* Now add the timer to the timer queue, making sure it is sorted in
-     increasing order of expiration. */
+     increasing order of expiration.  */
 
   for (timer_index = timer_list.first_timer;
        timer_index != NULL;
        timer_index = timer_index->next)
     {
       /* If the seconds field is greater or if it is the same, but the
-         microsecond field is greater. */
-      if ((timer_index->when.tv_sec > timer_ptr->when.tv_sec) ||
-	  ((timer_index->when.tv_sec == timer_ptr->when.tv_sec)
-	   && (timer_index->when.tv_usec > timer_ptr->when.tv_usec)))
+         microsecond field is greater.  */
+      if ((timer_index->when.tv_sec > timer_ptr->when.tv_sec)
+	  || ((timer_index->when.tv_sec == timer_ptr->when.tv_sec)
+	      && (timer_index->when.tv_usec > timer_ptr->when.tv_usec)))
 	break;
     }
 
@@ -1240,13 +1264,13 @@ create_timer (int milliseconds, timer_handler_func * proc, gdb_client_data clien
 }
 
 /* There is a chance that the creator of the timer wants to get rid of
-   it before it expires. */
+   it before it expires.  */
 void
 delete_timer (int id)
 {
   struct gdb_timer *timer_ptr, *prev_timer = NULL;
 
-  /* Find the entry for the given timer. */
+  /* Find the entry for the given timer.  */
 
   for (timer_ptr = timer_list.first_timer; timer_ptr != NULL;
        timer_ptr = timer_ptr->next)
@@ -1257,7 +1281,7 @@ delete_timer (int id)
 
   if (timer_ptr == NULL)
     return;
-  /* Get rid of the timer in the timer list. */
+  /* Get rid of the timer in the timer list.  */
   if (timer_ptr == timer_list.first_timer)
     timer_list.first_timer = timer_ptr->next;
   else
@@ -1288,16 +1312,16 @@ handle_timer_event (event_data dummy)
 
   while (timer_ptr != NULL)
     {
-      if ((timer_ptr->when.tv_sec > time_now.tv_sec) ||
-	  ((timer_ptr->when.tv_sec == time_now.tv_sec) &&
-	   (timer_ptr->when.tv_usec > time_now.tv_usec)))
+      if ((timer_ptr->when.tv_sec > time_now.tv_sec)
+	  || ((timer_ptr->when.tv_sec == time_now.tv_sec)
+	      && (timer_ptr->when.tv_usec > time_now.tv_usec)))
 	break;
 
-      /* Get rid of the timer from the beginning of the list. */
+      /* Get rid of the timer from the beginning of the list.  */
       timer_list.first_timer = timer_ptr->next;
       saved_timer = timer_ptr;
       timer_ptr = timer_ptr->next;
-      /* Call the procedure associated with that timer. */
+      /* Call the procedure associated with that timer.  */
       (*saved_timer->proc) (saved_timer->client_data);
       xfree (saved_timer);
     }
@@ -1305,12 +1329,12 @@ handle_timer_event (event_data dummy)
   gdb_notifier.timeout_valid = 0;
 }
 
-/* Check whether any timers in the timers queue are ready. If at least
+/* Check whether any timers in the timers queue are ready.  If at least
    one timer is ready, stick an event onto the event queue.  Even in
    case more than one timer is ready, one event is enough, because the
    handle_timer_event() will go through the timers list and call the
-   procedures associated with all that have expired. Update the
-   timeout for the select() or poll() as well. */
+   procedures associated with all that have expired.l Update the
+   timeout for the select() or poll() as well.  */
 static void
 poll_timers (void)
 {
@@ -1322,15 +1346,15 @@ poll_timers (void)
       gettimeofday (&time_now, NULL);
       delta.tv_sec = timer_list.first_timer->when.tv_sec - time_now.tv_sec;
       delta.tv_usec = timer_list.first_timer->when.tv_usec - time_now.tv_usec;
-      /* borrow? */
+      /* Borrow?  */
       if (delta.tv_usec < 0)
 	{
 	  delta.tv_sec -= 1;
 	  delta.tv_usec += 1000000;
 	}
 
-      /* Oops it expired already. Tell select / poll to return
-         immediately. (Cannot simply test if delta.tv_sec is negative
+      /* Oops it expired already.  Tell select / poll to return
+         immediately.  (Cannot simply test if delta.tv_sec is negative
          because time_t might be unsigned.)  */
       if (timer_list.first_timer->when.tv_sec < time_now.tv_sec
 	  || (timer_list.first_timer->when.tv_sec == time_now.tv_sec
@@ -1348,8 +1372,8 @@ poll_timers (void)
 	  async_queue_event (event_ptr, TAIL);
 	}
 
-      /* Now we need to update the timeout for select/ poll, because we
-         don't want to sit there while this timer is expiring. */
+      /* Now we need to update the timeout for select/ poll, because
+         we don't want to sit there while this timer is expiring.  */
       if (use_poll)
 	{
 #ifdef HAVE_POLL
