@@ -1,7 +1,7 @@
 /* Support for printing Fortran types for GDB, the GNU debugger.
 
    Copyright (C) 1986, 1988, 1989, 1991, 1993, 1994, 1995, 1996, 1998, 2000,
-   2001, 2002, 2003, 2006, 2007, 2008, 2009, 2010
+   2001, 2002, 2003, 2006, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    Contributed by Motorola.  Adapted from the C version by Farooq Butt
@@ -36,7 +36,7 @@
 #include "gdb_string.h"
 #include <errno.h>
 
-#if 0				/* Currently unused */
+#if 0				/* Currently unused.  */
 static void f_type_print_args (struct type *, struct ui_file *);
 #endif
 
@@ -52,7 +52,7 @@ void f_type_print_base (struct type *, struct ui_file *, int, int);
 /* LEVEL is the depth to indent lines by.  */
 
 void
-f_print_type (struct type *type, char *varstring, struct ui_file *stream,
+f_print_type (struct type *type, const char *varstring, struct ui_file *stream,
 	      int show, int level)
 {
   enum type_code code;
@@ -76,7 +76,7 @@ f_print_type (struct type *type, char *varstring, struct ui_file *stream,
       fputs_filtered (varstring, stream);
 
       /* For demangled function names, we have the arglist as part of the name,
-         so don't print an additional pair of ()'s */
+         so don't print an additional pair of ()'s.  */
 
       demangled_args = varstring[strlen (varstring) - 1] == ')'; 
       f_type_print_varspec_suffix (type, stream, show, 0, demangled_args, 0);
@@ -153,7 +153,7 @@ f_type_print_varspec_suffix (struct type *type, struct ui_file *stream,
 			     int arrayprint_recurse_level)
 {
   int upper_bound, lower_bound;
-  int retcode;
+
   /* No static variables are permitted as an error call may occur during
      execution of this function.  */
 
@@ -182,7 +182,7 @@ f_type_print_varspec_suffix (struct type *type, struct ui_file *stream,
 	fprintf_filtered (stream, "%d:", lower_bound);
 
       /* Make sure that, if we have an assumed size array, we
-         print out a warning and print the upperbound as '*' */
+         print out a warning and print the upperbound as '*'.  */
 
       if (TYPE_ARRAY_UPPER_BOUND_IS_UNDEFINED (type))
 	fprintf_filtered (stream, "*");
@@ -258,9 +258,7 @@ void
 f_type_print_base (struct type *type, struct ui_file *stream, int show,
 		   int level)
 {
-  int retcode;
   int upper_bound;
-
   int index;
 
   QUIT;
@@ -273,7 +271,7 @@ f_type_print_base (struct type *type, struct ui_file *stream, int show,
     }
 
   /* When SHOW is zero or less, and there is a valid type name, then always
-     just print the type name directly from the type. */
+     just print the type name directly from the type.  */
 
   if ((show <= 0) && (TYPE_NAME (type) != NULL))
     {
@@ -314,16 +312,16 @@ f_type_print_base (struct type *type, struct ui_file *stream, int show,
       break;
 
     case TYPE_CODE_ERROR:
-      fprintfi_filtered (level, stream, "<unknown type>");
+      fprintfi_filtered (level, stream, "%s", TYPE_ERROR_NAME (type));
       break;
 
     case TYPE_CODE_RANGE:
-      /* This should not occur */
+      /* This should not occur.  */
       fprintfi_filtered (level, stream, "<range type>");
       break;
 
     case TYPE_CODE_CHAR:
-      /* Override name "char" and make it "character" */
+      /* Override name "char" and make it "character".  */
       fprintfi_filtered (level, stream, "character");
       break;
 
@@ -339,7 +337,7 @@ f_type_print_base (struct type *type, struct ui_file *stream, int show,
       break;
 
     case TYPE_CODE_STRING:
-      /* Strings may have dynamic upperbounds (lengths) like arrays. */
+      /* Strings may have dynamic upperbounds (lengths) like arrays.  */
 
       if (TYPE_ARRAY_UPPER_BOUND_IS_UNDEFINED (type))
 	fprintfi_filtered (level, stream, "character*(*)");
@@ -372,12 +370,16 @@ f_type_print_base (struct type *type, struct ui_file *stream, int show,
       fputs_filtered (TYPE_TAG_NAME (type), stream);
       break;
 
+    case TYPE_CODE_MODULE:
+      fprintfi_filtered (level, stream, "module %s", TYPE_TAG_NAME (type));
+      break;
+
     default_case:
     default:
       /* Handle types not explicitly handled by the other cases,
          such as fundamental types.  For these, just print whatever
          the type name is, as recorded in the type itself.  If there
-         is no type name, then complain. */
+         is no type name, then complain.  */
       if (TYPE_NAME (type) != NULL)
 	fprintfi_filtered (level, stream, "%s", TYPE_NAME (type));
       else
