@@ -1,5 +1,5 @@
 /* Portable <regex.h>.
-   Copyright (C) 2000, 2001, 2003, 2007, 2008, 2009
+   Copyright (C) 2000, 2001, 2003, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -28,4 +28,8 @@
 # include <regex.h>
 #endif
 
+/* From utils.c.  */
+struct cleanup *make_regfree_cleanup (regex_t *);
+char *get_regcomp_error (int, regex_t *);
+
 #endif /* not GDB_REGEX_H */
