@@ -1,5 +1,6 @@
 /* Interface for common GDB/MI data
-   Copyright (C) 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -42,6 +43,9 @@ async_reason_lookup (enum async_reply_reason reason)
   return async_reason_string_lookup[reason];
 }
 
+/* Provide a prototype to silence -Wmissing-prototypes.  */
+extern initialize_file_ftype _initialize_gdb_mi_common;
+
 void
 _initialize_gdb_mi_common (void)
 {
