@@ -1,5 +1,6 @@
 /* Interface for common GDB/MI data
-   Copyright (C) 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -19,8 +20,9 @@
 #ifndef MI_COMMON_H
 #define MI_COMMON_H
 
-/* Represents the reason why GDB is sending an asynchronous command to the
-   front end.  NOTE: When modifing this, don't forget to update gdb.texinfo!  */
+/* Represents the reason why GDB is sending an asynchronous command to
+   the front end.  NOTE: When modifing this, don't forget to update
+   gdb.texinfo!  */
 enum async_reply_reason
 {
   EXEC_ASYNC_BREAKPOINT_HIT = 0,
@@ -41,4 +43,19 @@ enum async_reply_reason
 
 const char *async_reason_lookup (enum async_reply_reason reason);
 
+struct mi_interp
+{
+  /* MI's output channels */
+  struct ui_file *out;
+  struct ui_file *err;
+  struct ui_file *log;
+  struct ui_file *targ;
+  struct ui_file *event_channel;
+
+  /* This is the interpreter for the mi... */
+  struct interp *mi2_interp;
+  struct interp *mi1_interp;
+  struct interp *mi_interp;
+};
+
 #endif
