@@ -1,13 +1,13 @@
 /* Definitions for remote debugging interface for ROM monitors.
-   Copyright 1990, 1991, 1992, 1994, 1995, 1996, 1997, 1998, 1999, 2000
-   Free Software Foundation, Inc.
+   Copyright (C) 1990, 1991, 1992, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
+   2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
    Contributed by Cygnus Support. Written by Rob Savoye for Cygnus.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,10 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
- */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef MONITOR_H
 #define MONITOR_H
@@ -65,7 +62,8 @@ struct memrw_cmd
     char *cmdll;		/* Command for long long (64 bit) read/write */
     char *resp_delim;		/* String just prior to the desired value */
     char *term;			/* Terminating string to search for */
-    char *term_cmd;		/* String to get out of sub-mode (if necessary) */
+    char *term_cmd;		/* String to get out of sub-mode (if
+				   necessary) */
   };
 
 struct regrw_cmd
@@ -74,18 +72,20 @@ struct regrw_cmd
     char *resp_delim;		/* String (actually a regexp if getmem) just
 				   prior to the desired value */
     char *term;			/* Terminating string to search for */
-    char *term_cmd;		/* String to get out of sub-mode (if necessary) */
+    char *term_cmd;		/* String to get out of sub-mode (if
+				   necessary) */
   };
 
 struct monitor_ops
   {
     int flags;			/* See below */
-    char **init;		/* List of init commands.  NULL terminated. */
+    char **init;		/* List of init commands.  NULL terminated.  */
     char *cont;			/* continue command */
     char *step;			/* single step */
     char *stop;			/* Interrupt program string */
-    char *set_break;		/* set a breakpoint. If NULL, monitor implementation
-				   sets its own to_insert_breakpoint method. */
+    char *set_break;		/* set a breakpoint.  If NULL, monitor
+				   implementation sets its own
+				   to_insert_breakpoint method.  */
     char *clr_break;		/* clear a breakpoint */
     char *clr_all_break;	/* Clear all breakpoints */
     char *fill;			/* Memory fill cmd (addr len val) */
@@ -99,11 +99,13 @@ struct monitor_ops
        of registers that we can parse to supply
        GDB with the value of a register.  */
     char *dump_registers;	/* Command to dump all regs at once */
-    char *register_pattern;	/* Pattern that picks out register from reg dump */
-    void (*supply_register) (char *name, int namelen, char *val, int vallen);
+    char *register_pattern;	/* Pattern that picks out register
+				   from reg dump */
+    void (*supply_register) (struct regcache *regcache, char *name,
+			     int namelen, char *val, int vallen);
     void (*load_routine) (struct serial *desc, char *file,
 			  int hashmark);	/* Download routine */
-    int (*dumpregs) (void);	/* routine to dump all registers */
+    int (*dumpregs) (struct regcache *);	/* Dump all registers */
     int (*continue_hook) (void);	/* Emit the continue command */
     int (*wait_filter) (char *buf,	/* Maybe contains registers */
 			int bufmax,
@@ -126,11 +128,11 @@ struct monitor_ops
   };
 
 /* The monitor ops magic number, used to detect if an ops structure doesn't
-   have the right number of entries filled in. */
+   have the right number of entries filled in.  */
 
 #define MONITOR_OPS_MAGIC 600925
 
-/* Flag definitions. */
+/* Flag definitions.  */
 
 /* If set, then clear breakpoint command uses address, otherwise it
    uses an index returned by the monitor.  */
@@ -138,7 +140,7 @@ struct monitor_ops
 #define MO_CLR_BREAK_USES_ADDR 0x1
 
 /* If set, then memory fill command uses STARTADDR, ENDADDR+1, VALUE
-   as args, else it uses STARTADDR, LENGTH, VALUE as args. */
+   as args, else it uses STARTADDR, LENGTH, VALUE as args.  */
 
 #define MO_FILL_USES_ADDR 0x2
 
@@ -147,35 +149,35 @@ struct monitor_ops
 
 #define MO_NEED_REGDUMP_AFTER_CONT 0x4
 
-/* getmem needs start addr and end addr */
+/* getmem needs start addr and end addr.  */
 
 #define MO_GETMEM_NEEDS_RANGE 0x8
 
-/* getmem can only read one loc at a time */
+/* getmem can only read one loc at a time.  */
 
 #define MO_GETMEM_READ_SINGLE 0x10
 
-/* handle \r\n combinations */
+/* handle \r\n combinations.  */
 
 #define MO_HANDLE_NL 0x20
 
-/* don't expect echos in monitor_open */
+/* don't expect echos in monitor_open.  */
 
 #define MO_NO_ECHO_ON_OPEN 0x40
 
-/* If set, send break to stop monitor */
+/* If set, send break to stop monitor.  */
 
 #define MO_SEND_BREAK_ON_STOP 0x80
 
-/* If set, target sends an ACK after each S-record */
+/* If set, target sends an ACK after each S-record.  */
 
 #define MO_SREC_ACK 0x100
 
-/* Allow 0x prefix on addresses retured from monitor */
+/* Allow 0x prefix on addresses retured from monitor.  */
 
 #define MO_HEX_PREFIX 0x200
 
-/* Some monitors require a different command when starting a program */
+/* Some monitors require a different command when starting a program.  */
 
 #define MO_RUN_FIRST_TIME 0x400
 
@@ -183,7 +185,7 @@ struct monitor_ops
 
 #define MO_NO_ECHO_ON_SETMEM 0x800
 
-/* If set, then register store command expects value BEFORE regname */
+/* If set, then register store command expects value BEFORE regname.  */
 
 #define MO_REGISTER_VALUE_FIRST 0x1000
 
@@ -225,24 +227,25 @@ struct monitor_ops
 #define MO_PRINT_PROGRAM_OUTPUT 0x200000
 
 /* Some dump bytes commands align the first data with the preceeding
-   16 byte boundary. Some print blanks and start at the exactly the
-   requested boundary. */
+   16 byte boundary.  Some print blanks and start at the exactly the
+   requested boundary.  */
 
 #define MO_EXACT_DUMPADDR 0x400000
 
 /* Rather entering and exiting the write memory dialog for each word byte,
    we can save time by transferring the whole block without exiting
-   the memory editing mode. You only need to worry about this
+   the memory editing mode.  You only need to worry about this
    if you are doing memory downloading.
-   This engages a new write function registered with dcache.
- */
+   This engages a new write function registered with dcache.  */
+
 #define MO_HAS_BLOCKWRITES 0x800000
 
 #define SREC_SIZE 160
 
 extern void monitor_open (char *args, struct monitor_ops *ops, int from_tty);
 extern void monitor_close (int quitting);
-extern char *monitor_supply_register (int regno, char *valstr);
+extern char *monitor_supply_register (struct regcache *regcache,
+				      int regno, char *valstr);
 extern int monitor_expect (char *prompt, char *buf, int buflen);
 extern int monitor_expect_prompt (char *buf, int buflen);
 /* Note: The variable argument functions monitor_printf and
@@ -255,6 +258,6 @@ extern void monitor_write (char *buf, int buflen);
 extern int monitor_readchar (void);
 extern char *monitor_get_dev_name (void);
 extern void init_monitor_ops (struct target_ops *);
-extern int monitor_dump_reg_block (char *dump_cmd);
+extern int monitor_dump_reg_block (struct regcache *regcache, char *dump_cmd);
 
 #endif
