@@ -1,7 +1,8 @@
 /* Definitions for symbol file management in GDB.
 
    Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-   2002, 2003, 2004, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+   2002, 2003, 2004, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -22,7 +23,7 @@
 #define OBJFILES_H
 
 #include "gdb_obstack.h"	/* For obstack internals.  */
-#include "symfile.h"		/* For struct psymbol_allocation_list */
+#include "symfile.h"		/* For struct psymbol_allocation_list.  */
 #include "progspace.h"
 
 struct bcache;
@@ -48,7 +49,7 @@ struct objfile_data;
    the debugging information, where these values are the starting
    address (inclusive) and ending address (exclusive) of the
    instruction space in the executable which correspond to the
-   "startup file", I.E. crt0.o in most cases.  This file is assumed to
+   "startup file", i.e. crt0.o in most cases.  This file is assumed to
    be a startup file and frames with pc's inside it are treated as
    nonexistent.  Setting these variables is necessary so that
    backtraces do not fly off the bottom of the stack.
@@ -56,7 +57,7 @@ struct objfile_data;
    NOTE: cagney/2003-09-09: It turns out that this "traditional"
    method doesn't work.  Corinna writes: ``It turns out that the call
    to test for "inside entry file" destroys a meaningful backtrace
-   under some conditions.  E. g. the backtrace tests in the asm-source
+   under some conditions.  E.g. the backtrace tests in the asm-source
    testcase are broken for some targets.  In this test the functions
    are all implemented as part of one file and the testcase is not
    necessarily linked with a start file (depending on the target).
@@ -117,7 +118,7 @@ struct obj_section
     /* Objfile this section is part of.  */
     struct objfile *objfile;
 
-    /* True if this "overlay section" is mapped into an "overlay region". */
+    /* True if this "overlay section" is mapped into an "overlay region".  */
     int ovly_mapped;
   };
 
@@ -140,7 +141,7 @@ struct obj_section
 /* The "objstats" structure provides a place for gdb to record some
    interesting information about its internal state at runtime, on a
    per objfile basis, such as information about the number of symbols
-   read, size of string table (if any), etc. */
+   read, size of string table (if any), etc.  */
 
 struct objstats
   {
@@ -184,16 +185,18 @@ struct objfile
        struct list {struct list *next; struct objfile *objfile};
 
        where the list structure is completely maintained separately within
-       each gdb process. */
+       each gdb process.  */
 
     struct objfile *next;
 
-    /* The object file's name, tilde-expanded and absolute.
-       Malloc'd; free it if you free this struct.  */
+    /* The object file's name, tilde-expanded and absolute.  Malloc'd; free it
+       if you free this struct.  This pointer is never NULL.  */
 
     char *name;
 
-    /* Some flag bits for this objfile. */
+    CORE_ADDR addr_low;
+
+    /* Some flag bits for this objfile.  */
 
     unsigned short flags;
 
@@ -203,13 +206,13 @@ struct objfile
 
     /* Each objfile points to a linked list of symtabs derived from this file,
        one symtab structure for each compilation unit (source file).  Each link
-       in the symtab list contains a backpointer to this objfile. */
+       in the symtab list contains a backpointer to this objfile.  */
 
     struct symtab *symtabs;
 
     /* Each objfile points to a linked list of partial symtabs derived from
        this file, one partial symtab structure for each compilation unit
-       (source file). */
+       (source file).  */
 
     struct partial_symtab *psymtabs;
 
@@ -220,7 +223,7 @@ struct objfile
 
     struct addrmap *psymtabs_addrmap;
 
-    /* List of freed partial symtabs, available for re-use */
+    /* List of freed partial symtabs, available for re-use.  */
 
     struct partial_symtab *free_psymtabs;
 
@@ -242,16 +245,16 @@ struct objfile
     long mtime;
 
     /* Obstack to hold objects that should be freed when we load a new symbol
-       table from this object file. */
+       table from this object file.  */
 
     struct obstack objfile_obstack; 
 
     /* A byte cache where we can stash arbitrary "chunks" of bytes that
-       will not change. */
+       will not change.  */
 
-    struct bcache *psymbol_cache;	/* Byte cache for partial syms */
-    struct bcache *macro_cache;          /* Byte cache for macros */
-    struct bcache *filename_cache;	 /* Byte cache for file names.  */
+    struct psymbol_bcache *psymbol_cache; /* Byte cache for partial syms.  */
+    struct bcache *macro_cache;           /* Byte cache for macros.  */
+    struct bcache *filename_cache;	  /* Byte cache for file names.  */
 
     /* Hash table for mapping symbol names to demangled names.  Each
        entry in the hash table is actually two consecutive strings,
@@ -261,19 +264,20 @@ struct objfile
     struct htab *demangled_names_hash;
 
     /* Vectors of all partial symbols read in from file.  The actual data
-       is stored in the objfile_obstack. */
+       is stored in the objfile_obstack.  */
 
     struct psymbol_allocation_list global_psymbols;
     struct psymbol_allocation_list static_psymbols;
 
     /* Each file contains a pointer to an array of minimal symbols for all
-       global symbols that are defined within the file.  The array is terminated
-       by a "null symbol", one that has a NULL pointer for the name and a zero
-       value for the address.  This makes it easy to walk through the array
-       when passed a pointer to somewhere in the middle of it.  There is also
-       a count of the number of symbols, which does not include the terminating
-       null symbol.  The array itself, as well as all the data that it points
-       to, should be allocated on the objfile_obstack for this file. */
+       global symbols that are defined within the file.  The array is
+       terminated by a "null symbol", one that has a NULL pointer for the
+       name and a zero value for the address.  This makes it easy to walk
+       through the array when passed a pointer to somewhere in the middle
+       of it.  There is also a count of the number of symbols, which does
+       not include the terminating null symbol.  The array itself, as well
+       as all the data that it points to, should be allocated on the
+       objfile_obstack for this file.  */
 
     struct minimal_symbol *msymbols;
     int minimal_symbol_count;
@@ -288,20 +292,20 @@ struct objfile
     struct minimal_symbol *msymbol_demangled_hash[MINIMAL_SYMBOL_HASH_SIZE];
 
     /* Structure which keeps track of functions that manipulate objfile's
-       of the same type as this objfile.  I.E. the function to read partial
+       of the same type as this objfile.  I.e. the function to read partial
        symbols for example.  Note that this structure is in statically
        allocated memory, and is shared by all objfiles that use the
-       object module reader of this type. */
+       object module reader of this type.  */
 
-    struct sym_fns *sf;
+    const struct sym_fns *sf;
 
     /* The per-objfile information about the entry point, the scope (file/func)
-       containing the entry point, and the scope of the user's main() func. */
+       containing the entry point, and the scope of the user's main() func.  */
 
     struct entry_info ei;
 
     /* Information about stabs.  Will be filled in with a dbx_symfile_info
-       struct by those readers that need it. */
+       struct by those readers that need it.  */
     /* NOTE: cagney/2004-10-23: This has been replaced by per-objfile
        data points implemented using "data" and "num_data" below.  For
        an example of how to use this replacement, see "objfile_data"
@@ -340,12 +344,12 @@ struct objfile
     struct section_offsets *section_offsets;
     int num_sections;
 
-    /* Indexes in the section_offsets array. These are initialized by the
+    /* Indexes in the section_offsets array.  These are initialized by the
        *_symfile_offsets() family of functions (som_symfile_offsets,
-       xcoff_symfile_offsets, default_symfile_offsets). In theory they
+       xcoff_symfile_offsets, default_symfile_offsets).  In theory they
        should correspond to the section indexes used by bfd for the
-       current objfile. The exception to this for the time being is the
-       SOM version. */
+       current objfile.  The exception to this for the time being is the
+       SOM version.  */
 
     int sect_index_text;
     int sect_index_data;
@@ -375,14 +379,14 @@ struct objfile
     struct objfile *separate_debug_objfile;
 
     /* If this is a separate debug object, this is used as a link to the
-       actual executable objfile. */
+       actual executable objfile.  */
     struct objfile *separate_debug_objfile_backlink;
 
     /* If this is a separate debug object, this is a link to the next one
        for the same executable objfile.  */
     struct objfile *separate_debug_objfile_link;
 
-    /* Place to stash various statistics about this objfile */
+    /* Place to stash various statistics about this objfile.  */
       OBJSTATS;
 
     /* A symtab that the C++ code uses to stash special symbols
@@ -391,9 +395,15 @@ struct objfile
     /* FIXME/carlton-2003-06-27: Delete this in a few years once
        "possible namespace symbols" go away.  */
     struct symtab *cp_namespace_symtab;
+
+    /* A linked list of symbols created when reading template types or
+       function templates.  These symbols are not stored in any symbol
+       table, so we have to keep them here to relocate them
+       properly.  */
+    struct symbol *template_symbols;
   };
 
-/* Defines for the objfile flag word. */
+/* Defines for the objfile flag word.  */
 
 /* When an object file has its functions reordered (currently Irix-5.2
    shared libraries exhibit this behaviour), we will need an expensive
@@ -404,7 +414,7 @@ struct objfile
 #define OBJF_REORDERED	(1 << 0)	/* Functions are reordered */
 
 /* Distinguish between an objfile for a shared library and a "vanilla"
-   objfile. (If not set, the objfile may still actually be a solib.
+   objfile.  (If not set, the objfile may still actually be a solib.
    This can happen if the user created the objfile by using the
    add-symbol-file command.  GDB doesn't in that situation actually
    check whether the file is a solib.  Rather, the target's
@@ -413,7 +423,7 @@ struct objfile
 
 #define OBJF_SHARED     (1 << 1)	/* From a shared library */
 
-/* User requested that this objfile be read in it's entirety. */
+/* User requested that this objfile be read in it's entirety.  */
 
 #define OBJF_READNOW	(1 << 2)	/* Immediate full read */
 
@@ -422,12 +432,17 @@ struct objfile
    for run_command to remove old objfile entries which are no longer
    valid (i.e., are associated with an old inferior), but to preserve
    ones that the user explicitly loaded via the add-symbol-file
-   command. */
+   command.  */
 
 #define OBJF_USERLOADED	(1 << 3)	/* User loaded */
 
+/* Set if we have tried to read partial symtabs for this objfile.
+   This is used to allow lazy reading of partial symtabs.  */
+
+#define OBJF_PSYMTABS_READ (1 << 4)
+
 /* The object file that contains the runtime common minimal symbols
-   for SunOS4. Note that this objfile has no associated BFD.  */
+   for SunOS4.  Note that this objfile has no associated BFD.  */
 
 extern struct objfile *rt_common_objfile;
 
@@ -442,7 +457,7 @@ extern struct objfile *rt_common_objfile;
    objfile, and when we are done, we set it back to NULL, to ensure that we
    never put a type someplace other than where we are expecting to put it.
    FIXME:  Maybe we should review the entire type handling system and
-   see if there is a better way to avoid this problem. */
+   see if there is a better way to avoid this problem.  */
 
 extern struct objfile *current_objfile;
 
@@ -497,8 +512,8 @@ extern void objfiles_changed (void);
 
 /* This operation deletes all objfile entries that represent solibs that
    weren't explicitly loaded by the user, via e.g., the add-symbol-file
-   command.
- */
+   command.  */
+
 extern void objfile_purge_solibs (void);
 
 /* Functions for dealing with the minimal symbol table, really a misc
@@ -533,6 +548,7 @@ extern void *objfile_data (struct objfile *objfile,
 
 extern struct bfd *gdb_bfd_ref (struct bfd *abfd);
 extern void gdb_bfd_unref (struct bfd *abfd);
+extern int gdb_bfd_close_or_warn (struct bfd *abfd);
 
 
 /* Traverse all object files in the current program space.
@@ -564,11 +580,6 @@ extern void gdb_bfd_unref (struct bfd *abfd);
 #define	ALL_OBJFILE_SYMTABS(objfile, s) \
     for ((s) = (objfile) -> symtabs; (s) != NULL; (s) = (s) -> next)
 
-/* Traverse all psymtabs in one objfile.  */
-
-#define	ALL_OBJFILE_PSYMTABS(objfile, p) \
-    for ((p) = (objfile) -> psymtabs; (p) != NULL; (p) = (p) -> next)
-
 /* Traverse all minimal symbols in one objfile.  */
 
 #define	ALL_OBJFILE_MSYMBOLS(objfile, m) \
@@ -599,17 +610,6 @@ extern void gdb_bfd_unref (struct bfd *abfd);
     ALL_OBJFILE_SYMTABS (objfile, s)			\
       if ((s)->primary)
 
-/* Traverse all psymtabs in all objfiles in the current symbol
-   space.  */
-
-#define	ALL_PSYMTABS(objfile, p) \
-  ALL_OBJFILES (objfile)	 \
-    ALL_OBJFILE_PSYMTABS (objfile, p)
-
-#define ALL_PSPACE_PSYMTABS(ss, objfile, p)		\
-  ALL_PSPACE_OBJFILES (ss, objfile)			\
-    ALL_OBJFILE_PSYMTABS (objfile, p)
-
 /* Traverse all minimal symbols in all objfiles in the current symbol
    space.  */
 
@@ -620,28 +620,65 @@ extern void gdb_bfd_unref (struct bfd *abfd);
 #define ALL_OBJFILE_OSECTIONS(objfile, osect)	\
   for (osect = objfile->sections; osect < objfile->sections_end; osect++)
 
-#define ALL_OBJSECTIONS(objfile, osect)		\
-  ALL_OBJFILES (objfile)			\
-    ALL_OBJFILE_OSECTIONS (objfile, osect)
+/* Traverse all obj_sections in all objfiles in the current program
+   space.
+
+   Note that this detects a "break" in the inner loop, and exits
+   immediately from the outer loop as well, thus, client code doesn't
+   need to know that this is implemented with a double for.  The extra
+   hair is to make sure that a "break;" stops the outer loop iterating
+   as well, and both OBJFILE and OSECT are left unmodified:
+
+    - The outer loop learns about the inner loop's end condition, and
+      stops iterating if it detects the inner loop didn't reach its
+      end.  In other words, the outer loop keeps going only if the
+      inner loop reached its end cleanly [(osect) ==
+      (objfile)->sections_end].
+
+    - OSECT is initialized in the outer loop initialization
+      expressions, such as if the inner loop has reached its end, so
+      the check mentioned above succeeds the first time.
+
+    - The trick to not clearing OBJFILE on a "break;" is, in the outer
+      loop's loop expression, advance OBJFILE, but iff the inner loop
+      reached its end.  If not, there was a "break;", so leave OBJFILE
+      as is; the outer loop's conditional will break immediately as
+      well (as OSECT will be different from OBJFILE->sections_end).  */
+
+#define ALL_OBJSECTIONS(objfile, osect)					\
+  for ((objfile) = current_program_space->objfiles,			\
+	 (objfile) != NULL ? ((osect) = (objfile)->sections_end) : 0;	\
+       (objfile) != NULL						\
+	 && (osect) == (objfile)->sections_end;				\
+       ((osect) == (objfile)->sections_end				\
+	? ((objfile) = (objfile)->next,					\
+	   (objfile) != NULL ? (osect) = (objfile)->sections_end : 0)	\
+	: 0))								\
+    for ((osect) = (objfile)->sections;					\
+	 (osect) < (objfile)->sections_end;				\
+	 (osect)++)
 
 #define SECT_OFF_DATA(objfile) \
      ((objfile->sect_index_data == -1) \
-      ? (internal_error (__FILE__, __LINE__, _("sect_index_data not initialized")), -1) \
+      ? (internal_error (__FILE__, __LINE__, \
+			 _("sect_index_data not initialized")), -1)	\
       : objfile->sect_index_data)
 
 #define SECT_OFF_RODATA(objfile) \
      ((objfile->sect_index_rodata == -1) \
-      ? (internal_error (__FILE__, __LINE__, _("sect_index_rodata not initialized")), -1) \
+      ? (internal_error (__FILE__, __LINE__, \
+			 _("sect_index_rodata not initialized")), -1)	\
       : objfile->sect_index_rodata)
 
 #define SECT_OFF_TEXT(objfile) \
      ((objfile->sect_index_text == -1) \
-      ? (internal_error (__FILE__, __LINE__, _("sect_index_text not initialized")), -1) \
+      ? (internal_error (__FILE__, __LINE__, \
+			 _("sect_index_text not initialized")), -1)	\
       : objfile->sect_index_text)
 
 /* Sometimes the .bss section is missing from the objfile, so we don't
-   want to die here. Let the users of SECT_OFF_BSS deal with an
-   uninitialized section index. */
+   want to die here.  Let the users of SECT_OFF_BSS deal with an
+   uninitialized section index.  */
 #define SECT_OFF_BSS(objfile) (objfile)->sect_index_bss
 
 /* Answer whether there is more than one object file loaded.  */
