@@ -1,5 +1,5 @@
 /* Support for printing Pascal types for GDB, the GNU debugger.
-   Copyright (C) 2000, 2001, 2002, 2006, 2007, 2008, 2009, 2010
+   Copyright (C) 2000, 2001, 2002, 2006, 2007, 2008, 2009, 2010, 2011
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -36,11 +36,14 @@
 #include <errno.h>
 #include <ctype.h>
 
-static void pascal_type_print_varspec_suffix (struct type *, struct ui_file *, int, int, int);
+static void pascal_type_print_varspec_suffix (struct type *, struct ui_file *,
+					      int, int, int);
 
-static void pascal_type_print_derivation_info (struct ui_file *, struct type *);
+static void pascal_type_print_derivation_info (struct ui_file *,
+					       struct type *);
 
-void pascal_type_print_varspec_prefix (struct type *, struct ui_file *, int, int);
+void pascal_type_print_varspec_prefix (struct type *, struct ui_file *,
+				       int, int);
 
 
 /* LEVEL is the depth to indent lines by.  */
@@ -80,7 +83,7 @@ pascal_print_type (struct type *type, const char *varstring,
 
   pascal_type_print_base (type, stream, show, level);
   /* For demangled function names, we have the arglist as part of the name,
-     so don't print an additional pair of ()'s */
+     so don't print an additional pair of ()'s.  */
 
   demangled_args = varstring ? strchr (varstring, '(') != NULL : 0;
   pascal_type_print_varspec_suffix (type, stream, show, 0, demangled_args);
@@ -104,7 +107,7 @@ pascal_print_typedef (struct type *type, struct symbol *new_symbol,
 
 /* If TYPE is a derived type, then print out derivation information.
    Print only the actual base classes of this type, not the base classes
-   of the base classes.  I.E.  for the derivation hierarchy:
+   of the base classes.  I.e. for the derivation hierarchy:
 
    class A { int a; };
    class B : public A {int b; };
@@ -124,7 +127,7 @@ pascal_print_typedef (struct type *type, struct symbol *new_symbol,
    }
 
    In general, gdb should try to print the types as closely as possible to
-   the form that they appear in the source code. */
+   the form that they appear in the source code.  */
 
 static void
 pascal_type_print_derivation_info (struct ui_file *stream, struct type *type)
@@ -165,14 +168,15 @@ pascal_type_print_method_args (char *physname, char *methodname,
 
   if (physname && (*physname != 0))
     {
-      int i = 0;
-      int len = 0;
-      char storec;
-      char *argname;
       fputs_filtered (" (", stream);
-      /* we must demangle this */
+      /* We must demangle this.  */
       while (isdigit (physname[0]))
 	{
+	  int len = 0;
+	  int i;
+	  char storec;
+	  char *argname;
+
 	  while (isdigit (physname[len]))
 	    {
 	      len++;
@@ -218,7 +222,8 @@ pascal_type_print_varspec_prefix (struct type *type, struct ui_file *stream,
     case TYPE_CODE_PTR:
       fprintf_filtered (stream, "^");
       pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 1);
-      break;			/* pointer should be handled normally in pascal */
+      break;			/* Pointer should be handled normally
+				   in pascal.  */
 
     case TYPE_CODE_METHOD:
       if (passed_a_ptr)
@@ -235,7 +240,8 @@ pascal_type_print_varspec_prefix (struct type *type, struct ui_file *stream,
       if (passed_a_ptr)
 	{
 	  fprintf_filtered (stream, " ");
-	  pascal_type_print_base (TYPE_DOMAIN_TYPE (type), stream, 0, passed_a_ptr);
+	  pascal_type_print_base (TYPE_DOMAIN_TYPE (type),
+				  stream, 0, passed_a_ptr);
 	  fprintf_filtered (stream, "::");
 	}
       break;
@@ -313,12 +319,13 @@ pascal_print_func_args (struct type *type, struct ui_file *stream)
 	  fputs_filtered (", ", stream);
 	  wrap_here ("    ");
 	}
-      /*  can we find if it is a var parameter ??
+      /*  Can we find if it is a var parameter ??
          if ( TYPE_FIELD(type, i) == )
          {
          fprintf_filtered (stream, "var ");
          } */
-      pascal_print_type (TYPE_FIELD_TYPE (type, i), ""	/* TYPE_FIELD_NAME seems invalid ! */
+      pascal_print_type (TYPE_FIELD_TYPE (type, i), ""	/* TYPE_FIELD_NAME
+							   seems invalid!  */
 			 ,stream, -1, 0);
     }
   if (len)
@@ -360,7 +367,8 @@ pascal_type_print_varspec_suffix (struct type *type, struct ui_file *stream,
       if (TYPE_CODE (TYPE_TARGET_TYPE (type)) != TYPE_CODE_VOID)
 	{
 	  fprintf_filtered (stream, " : ");
-	  pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 0);
+	  pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type),
+					    stream, 0, 0);
 	  pascal_type_print_base (TYPE_TARGET_TYPE (type), stream, show, 0);
 	  pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0,
 					    passed_a_ptr, 0);
@@ -369,7 +377,8 @@ pascal_type_print_varspec_suffix (struct type *type, struct ui_file *stream,
 
     case TYPE_CODE_PTR:
     case TYPE_CODE_REF:
-      pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0, 1, 0);
+      pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type),
+					stream, 0, 1, 0);
       break;
 
     case TYPE_CODE_FUNC:
@@ -380,7 +389,8 @@ pascal_type_print_varspec_suffix (struct type *type, struct ui_file *stream,
       if (TYPE_CODE (TYPE_TARGET_TYPE (type)) != TYPE_CODE_VOID)
 	{
 	  fprintf_filtered (stream, " : ");
-	  pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 0);
+	  pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type),
+					    stream, 0, 0);
 	  pascal_type_print_base (TYPE_TARGET_TYPE (type), stream, show, 0);
 	  pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0,
 					    passed_a_ptr, 0);
@@ -450,7 +460,8 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
     }
 
   /* void pointer */
-  if ((TYPE_CODE (type) == TYPE_CODE_PTR) && (TYPE_CODE (TYPE_TARGET_TYPE (type)) == TYPE_CODE_VOID))
+  if ((TYPE_CODE (type) == TYPE_CODE_PTR)
+      && (TYPE_CODE (TYPE_TARGET_TYPE (type)) == TYPE_CODE_VOID))
     {
       fputs_filtered (TYPE_NAME (type) ? TYPE_NAME (type) : "pointer",
 		      stream);
@@ -479,9 +490,12 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
       break;
 
     case TYPE_CODE_ARRAY:
-      /* pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type), stream, 0, 0);
-         pascal_type_print_base (TYPE_TARGET_TYPE (type), stream, show, level);
-         pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type), stream, 0, 0, 0); */
+      /* pascal_type_print_varspec_prefix (TYPE_TARGET_TYPE (type),
+	                                   stream, 0, 0);
+         pascal_type_print_base (TYPE_TARGET_TYPE (type),
+	                         stream, show, level);
+         pascal_type_print_varspec_suffix (TYPE_TARGET_TYPE (type),
+	                                   stream, 0, 0, 0); */
       pascal_print_type (TYPE_TARGET_TYPE (type), NULL, stream, 0, 0);
       break;
 
@@ -489,7 +503,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
     case TYPE_CODE_METHOD:
       /*
          pascal_type_print_base (TYPE_TARGET_TYPE (type), stream, show, level);
-         only after args !! */
+         only after args !!  */
       break;
     case TYPE_CODE_STRUCT:
       if (TYPE_TAG_NAME (type) != NULL)
@@ -538,7 +552,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 
 	  /* Start off with no specific section type, so we can print
 	     one for the first field we find, and use that section type
-	     thereafter until we find another type. */
+	     thereafter until we find another type.  */
 
 	  section_type = s_none;
 
@@ -555,7 +569,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 		continue;
 
 	      /* If this is a pascal object or class we can print the
-	         various section labels. */
+	         various section labels.  */
 
 	      if (HAVE_CPLUS_STRUCT (type))
 		{
@@ -606,12 +620,12 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 	      fprintf_filtered (stream, ";\n");
 	    }
 
-	  /* If there are both fields and methods, put a space between. */
+	  /* If there are both fields and methods, put a space between.  */
 	  len = TYPE_NFN_FIELDS (type);
 	  if (len && section_type != s_none)
 	    fprintf_filtered (stream, "\n");
 
-	  /* Pbject pascal: print out the methods */
+	  /* Object pascal: print out the methods.  */
 
 	  for (i = 0; i < len; i++)
 	    {
@@ -621,7 +635,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 
 	      /* this is GNU C++ specific
 	         how can we know constructor/destructor?
-	         It might work for GNU pascal */
+	         It might work for GNU pascal.  */
 	      for (j = 0; j < len2; j++)
 		{
 		  char *physname = TYPE_FN_FIELD_PHYSNAME (f, j);
@@ -685,7 +699,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 		    {
 		      fprintf_filtered (stream, "procedure ");
 		    }
-		  /* this does not work, no idea why !! */
+		  /* This does not work, no idea why !!  */
 
 		  pascal_type_print_method_args (physname,
 						 method_name,
@@ -717,7 +731,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 	    fputs_filtered (" ", stream);
 	}
       /* enum is just defined by
-         type enume_name = (enum_member1,enum_member2,...) */
+         type enume_name = (enum_member1,enum_member2,...)  */
       fprintf_filtered (stream, " = ");
       wrap_here ("    ");
       if (show < 0)
@@ -740,7 +754,8 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
 	      fputs_filtered (TYPE_FIELD_NAME (type, i), stream);
 	      if (lastval != TYPE_FIELD_BITPOS (type, i))
 		{
-		  fprintf_filtered (stream, " := %d", TYPE_FIELD_BITPOS (type, i));
+		  fprintf_filtered (stream,
+				    " := %d", TYPE_FIELD_BITPOS (type, i));
 		  lastval = TYPE_FIELD_BITPOS (type, i);
 		}
 	      lastval++;
@@ -761,7 +776,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
       fprintf_filtered (stream, "%s", TYPE_ERROR_NAME (type));
       break;
 
-      /* this probably does not work for enums */
+      /* this probably does not work for enums.  */
     case TYPE_CODE_RANGE:
       {
 	struct type *target = TYPE_TARGET_TYPE (type);
@@ -790,7 +805,7 @@ pascal_type_print_base (struct type *type, struct ui_file *stream, int show,
       /* Handle types not explicitly handled by the other cases,
          such as fundamental types.  For these, just print whatever
          the type name is, as recorded in the type itself.  If there
-         is no type name, then complain. */
+         is no type name, then complain.  */
       if (TYPE_NAME (type) != NULL)
 	{
 	  fputs_filtered (TYPE_NAME (type), stream);
