@@ -1,8 +1,8 @@
 /* Do various things to symbol tables (other than lookup), for GDB.
 
    Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
-   1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2007, 2008, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -23,6 +23,7 @@
 #include "symtab.h"
 #include "gdbtypes.h"
 #include "bfd.h"
+#include "filenames.h"
 #include "symfile.h"
 #include "objfiles.h"
 #include "breakpoint.h"
@@ -39,6 +40,8 @@
 #include "gdb_string.h"
 #include "readline/readline.h"
 
+#include "psymtab.h"
+
 #ifndef DEV_TTY
 #define DEV_TTY "/dev/tty"
 #endif
@@ -58,19 +61,12 @@ FILE *std_err;
 static void dump_symtab (struct objfile *, struct symtab *,
 			 struct ui_file *);
 
-static void dump_psymtab (struct objfile *, struct partial_symtab *,
-			  struct ui_file *);
-
 static void dump_msymbols (struct objfile *, struct ui_file *);
 
 static void dump_objfile (struct objfile *);
 
 static int block_depth (struct block *);
 
-static void print_partial_symbols (struct gdbarch *,
-				   struct partial_symbol **, int,
-				   char *, struct ui_file *);
-
 void _initialize_symmisc (void);
 
 struct print_symbol_args
@@ -93,9 +89,6 @@ static int print_symbol (void *);
 void
 free_symtab (struct symtab *s)
 {
-  int i, n;
-  struct blockvector *bv;
-
   switch (s->free_code)
     {
     case free_nothing:
@@ -117,7 +110,7 @@ free_symtab (struct symtab *s)
   if (s->free_func != NULL)
     s->free_func (s);
 
-  /* Free source-related stuff */
+  /* Free source-related stuff.  */
   if (s->line_charpos != NULL)
     xfree (s->line_charpos);
   if (s->fullname != NULL)
@@ -138,7 +131,8 @@ print_symbol_bcache_statistics (void)
     ALL_PSPACE_OBJFILES (pspace, objfile)
   {
     printf_filtered (_("Byte cache statistics for '%s':\n"), objfile->name);
-    print_bcache_statistics (objfile->psymbol_cache, "partial symbol cache");
+    print_bcache_statistics (psymbol_bcache_get_bcache (objfile->psymbol_cache),
+                             "partial symbol cache");
     print_bcache_statistics (objfile->macro_cache, "preprocessor macro cache");
     print_bcache_statistics (objfile->filename_cache, "file name cache");
   }
@@ -151,7 +145,6 @@ print_objfile_statistics (void)
   struct program_space *pspace;
   struct objfile *objfile;
   struct symtab *s;
-  struct partial_symtab *ps;
   int i, linetables, blockvectors;
 
   immediate_quit++;
@@ -174,13 +167,8 @@ print_objfile_statistics (void)
     if (OBJSTAT (objfile, n_types) > 0)
       printf_filtered (_("  Number of \"types\" defined: %d\n"),
 		       OBJSTAT (objfile, n_types));
-    i = 0;
-    ALL_OBJFILE_PSYMTABS (objfile, ps)
-      {
-        if (ps->readin == 0)
-          i++;
-      }
-    printf_filtered (_("  Number of psym tables (not yet expanded): %d\n"), i);
+    if (objfile->sf)
+      objfile->sf->qf->print_stats (objfile);
     i = linetables = blockvectors = 0;
     ALL_OBJFILE_SYMTABS (objfile, s)
       {
@@ -202,7 +190,8 @@ print_objfile_statistics (void)
     printf_filtered (_("  Total memory used for objfile obstack: %d\n"),
 		     obstack_memory_used (&objfile->objfile_obstack));
     printf_filtered (_("  Total memory used for psymbol cache: %d\n"),
-		     bcache_memory_used (objfile->psymbol_cache));
+		     bcache_memory_used (psymbol_bcache_get_bcache
+		                          (objfile->psymbol_cache)));
     printf_filtered (_("  Total memory used for macro cache: %d\n"),
 		     bcache_memory_used (objfile->macro_cache));
     printf_filtered (_("  Total memory used for file name cache: %d\n"),
@@ -215,7 +204,6 @@ static void
 dump_objfile (struct objfile *objfile)
 {
   struct symtab *symtab;
-  struct partial_symtab *psymtab;
 
   printf_filtered ("\nObject file %s:  ", objfile->name);
   printf_filtered ("Objfile at ");
@@ -225,25 +213,8 @@ dump_objfile (struct objfile *objfile)
   printf_filtered (", %d minsyms\n\n",
 		   objfile->minimal_symbol_count);
 
-  if (objfile->psymtabs)
-    {
-      printf_filtered ("Psymtabs:\n");
-      for (psymtab = objfile->psymtabs;
-	   psymtab != NULL;
-	   psymtab = psymtab->next)
-	{
-	  printf_filtered ("%s at ",
-			   psymtab->filename);
-	  gdb_print_host_address (psymtab, gdb_stdout);
-	  printf_filtered (", ");
-	  if (psymtab->objfile != objfile)
-	    {
-	      printf_filtered ("NOT ON CHAIN!  ");
-	    }
-	  wrap_here ("  ");
-	}
-      printf_filtered ("\n\n");
-    }
+  if (objfile->sf)
+    objfile->sf->qf->dump (objfile);
 
   if (objfile->symtabs)
     {
@@ -294,6 +265,9 @@ dump_msymbols (struct objfile *objfile, struct ui_file *outfile)
 	case mst_text:
 	  ms_type = 'T';
 	  break;
+	case mst_text_gnu_ifunc:
+	  ms_type = 'i';
+	  break;
 	case mst_solib_trampoline:
 	  ms_type = 'S';
 	  break;
@@ -344,76 +318,6 @@ dump_msymbols (struct objfile *objfile, struct ui_file *outfile)
   fprintf_filtered (outfile, "\n");
 }
 
-static void
-dump_psymtab (struct objfile *objfile, struct partial_symtab *psymtab,
-	      struct ui_file *outfile)
-{
-  struct gdbarch *gdbarch = get_objfile_arch (objfile);
-  int i;
-
-  fprintf_filtered (outfile, "\nPartial symtab for source file %s ",
-		    psymtab->filename);
-  fprintf_filtered (outfile, "(object ");
-  gdb_print_host_address (psymtab, outfile);
-  fprintf_filtered (outfile, ")\n\n");
-  fprintf_unfiltered (outfile, "  Read from object file %s (",
-		      objfile->name);
-  gdb_print_host_address (objfile, outfile);
-  fprintf_unfiltered (outfile, ")\n");
-
-  if (psymtab->readin)
-    {
-      fprintf_filtered (outfile,
-			"  Full symtab was read (at ");
-      gdb_print_host_address (psymtab->symtab, outfile);
-      fprintf_filtered (outfile, " by function at ");
-      gdb_print_host_address (psymtab->read_symtab, outfile);
-      fprintf_filtered (outfile, ")\n");
-    }
-
-  fprintf_filtered (outfile, "  Relocate symbols by ");
-  for (i = 0; i < psymtab->objfile->num_sections; ++i)
-    {
-      if (i != 0)
-	fprintf_filtered (outfile, ", ");
-      wrap_here ("    ");
-      fputs_filtered (paddress (gdbarch,
-				ANOFFSET (psymtab->section_offsets, i)),
-		      outfile);
-    }
-  fprintf_filtered (outfile, "\n");
-
-  fprintf_filtered (outfile, "  Symbols cover text addresses ");
-  fputs_filtered (paddress (gdbarch, psymtab->textlow), outfile);
-  fprintf_filtered (outfile, "-");
-  fputs_filtered (paddress (gdbarch, psymtab->texthigh), outfile);
-  fprintf_filtered (outfile, "\n");
-  fprintf_filtered (outfile, "  Depends on %d other partial symtabs.\n",
-		    psymtab->number_of_dependencies);
-  for (i = 0; i < psymtab->number_of_dependencies; i++)
-    {
-      fprintf_filtered (outfile, "    %d ", i);
-      gdb_print_host_address (psymtab->dependencies[i], outfile);
-      fprintf_filtered (outfile, " %s\n",
-			psymtab->dependencies[i]->filename);
-    }
-  if (psymtab->n_global_syms > 0)
-    {
-      print_partial_symbols (gdbarch,
-			     objfile->global_psymbols.list
-			     + psymtab->globals_offset,
-			     psymtab->n_global_syms, "Global", outfile);
-    }
-  if (psymtab->n_static_syms > 0)
-    {
-      print_partial_symbols (gdbarch,
-			     objfile->static_psymbols.list
-			     + psymtab->statics_offset,
-			     psymtab->n_static_syms, "Static", outfile);
-    }
-  fprintf_filtered (outfile, "\n");
-}
-
 static void
 dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
 	       struct ui_file *outfile)
@@ -421,7 +325,7 @@ dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
   struct gdbarch *gdbarch = get_objfile_arch (objfile);
   int i;
   struct dict_iterator iter;
-  int len, blen;
+  int len;
   struct linetable *l;
   struct blockvector *bv;
   struct symbol *sym;
@@ -435,7 +339,8 @@ dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
   fprintf_filtered (outfile, "Read from object file %s (", objfile->name);
   gdb_print_host_address (objfile, outfile);
   fprintf_filtered (outfile, ")\n");
-  fprintf_filtered (outfile, "Language: %s\n", language_str (symtab->language));
+  fprintf_filtered (outfile, "Language: %s\n",
+		    language_str (symtab->language));
 
   /* First print the line table.  */
   l = LINETABLE (symtab);
@@ -451,7 +356,7 @@ dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
 	}
     }
   /* Now print the block info, but only for primary symtabs since we will
-     print lots of duplicate info otherwise. */
+     print lots of duplicate info otherwise.  */
   if (symtab->primary)
     {
       fprintf_filtered (outfile, "\nBlockvector:\n\n");
@@ -493,6 +398,7 @@ dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
 	  ALL_BLOCK_SYMBOLS (b, iter, sym)
 	    {
 	      struct print_symbol_args s;
+
 	      s.gdbarch = gdbarch;
 	      s.symbol = sym;
 	      s.depth = depth + 1;
@@ -547,8 +453,8 @@ maintenance_print_symbols (char *args, int from_tty)
 
   if (args == NULL)
     {
-      error (_("\
-Arguments missing: an output file name and an optional symbol file name"));
+      error (_("Arguments missing: an output file name "
+	       "and an optional symbol file name"));
     }
   argv = gdb_buildargv (args);
   cleanups = make_cleanup_freeargv (argv);
@@ -556,7 +462,7 @@ Arguments missing: an output file name and an optional symbol file name"));
   if (argv[0] != NULL)
     {
       filename = argv[0];
-      /* If a second arg is supplied, it is a source file name to match on */
+      /* If a second arg is supplied, it is a source file name to match on.  */
       if (argv[1] != NULL)
 	{
 	  symname = argv[1];
@@ -573,7 +479,7 @@ Arguments missing: an output file name and an optional symbol file name"));
 
   immediate_quit++;
   ALL_SYMTABS (objfile, s)
-    if (symname == NULL || strcmp (symname, s->filename) == 0)
+    if (symname == NULL || filename_cmp (symname, s->filename) == 0)
     dump_symtab (objfile, s, outfile);
   immediate_quit--;
   do_cleanups (cleanups);
@@ -653,6 +559,7 @@ print_symbol (void *args)
 	  {
 	    unsigned i;
 	    struct type *type = check_typedef (SYMBOL_TYPE (symbol));
+
 	    fprintf_filtered (outfile, "const %u hex bytes:",
 			      TYPE_LENGTH (type));
 	    for (i = 0; i < TYPE_LENGTH (type); i++)
@@ -749,142 +656,6 @@ print_symbol (void *args)
   return 1;
 }
 
-void
-maintenance_print_psymbols (char *args, int from_tty)
-{
-  char **argv;
-  struct ui_file *outfile;
-  struct cleanup *cleanups;
-  char *symname = NULL;
-  char *filename = DEV_TTY;
-  struct objfile *objfile;
-  struct partial_symtab *ps;
-
-  dont_repeat ();
-
-  if (args == NULL)
-    {
-      error (_("print-psymbols takes an output file name and optional symbol file name"));
-    }
-  argv = gdb_buildargv (args);
-  cleanups = make_cleanup_freeargv (argv);
-
-  if (argv[0] != NULL)
-    {
-      filename = argv[0];
-      /* If a second arg is supplied, it is a source file name to match on */
-      if (argv[1] != NULL)
-	{
-	  symname = argv[1];
-	}
-    }
-
-  filename = tilde_expand (filename);
-  make_cleanup (xfree, filename);
-
-  outfile = gdb_fopen (filename, FOPEN_WT);
-  if (outfile == 0)
-    perror_with_name (filename);
-  make_cleanup_ui_file_delete (outfile);
-
-  immediate_quit++;
-  ALL_PSYMTABS (objfile, ps)
-    if (symname == NULL || strcmp (symname, ps->filename) == 0)
-    dump_psymtab (objfile, ps, outfile);
-  immediate_quit--;
-  do_cleanups (cleanups);
-}
-
-static void
-print_partial_symbols (struct gdbarch *gdbarch,
-		       struct partial_symbol **p, int count, char *what,
-		       struct ui_file *outfile)
-{
-  fprintf_filtered (outfile, "  %s partial symbols:\n", what);
-  while (count-- > 0)
-    {
-      fprintf_filtered (outfile, "    `%s'", SYMBOL_LINKAGE_NAME (*p));
-      if (SYMBOL_DEMANGLED_NAME (*p) != NULL)
-	{
-	  fprintf_filtered (outfile, "  `%s'", SYMBOL_DEMANGLED_NAME (*p));
-	}
-      fputs_filtered (", ", outfile);
-      switch (SYMBOL_DOMAIN (*p))
-	{
-	case UNDEF_DOMAIN:
-	  fputs_filtered ("undefined domain, ", outfile);
-	  break;
-	case VAR_DOMAIN:
-	  /* This is the usual thing -- don't print it */
-	  break;
-	case STRUCT_DOMAIN:
-	  fputs_filtered ("struct domain, ", outfile);
-	  break;
-	case LABEL_DOMAIN:
-	  fputs_filtered ("label domain, ", outfile);
-	  break;
-	default:
-	  fputs_filtered ("<invalid domain>, ", outfile);
-	  break;
-	}
-      switch (SYMBOL_CLASS (*p))
-	{
-	case LOC_UNDEF:
-	  fputs_filtered ("undefined", outfile);
-	  break;
-	case LOC_CONST:
-	  fputs_filtered ("constant int", outfile);
-	  break;
-	case LOC_STATIC:
-	  fputs_filtered ("static", outfile);
-	  break;
-	case LOC_REGISTER:
-	  fputs_filtered ("register", outfile);
-	  break;
-	case LOC_ARG:
-	  fputs_filtered ("pass by value", outfile);
-	  break;
-	case LOC_REF_ARG:
-	  fputs_filtered ("pass by reference", outfile);
-	  break;
-	case LOC_REGPARM_ADDR:
-	  fputs_filtered ("register address parameter", outfile);
-	  break;
-	case LOC_LOCAL:
-	  fputs_filtered ("stack parameter", outfile);
-	  break;
-	case LOC_TYPEDEF:
-	  fputs_filtered ("type", outfile);
-	  break;
-	case LOC_LABEL:
-	  fputs_filtered ("label", outfile);
-	  break;
-	case LOC_BLOCK:
-	  fputs_filtered ("function", outfile);
-	  break;
-	case LOC_CONST_BYTES:
-	  fputs_filtered ("constant bytes", outfile);
-	  break;
-	case LOC_UNRESOLVED:
-	  fputs_filtered ("unresolved", outfile);
-	  break;
-	case LOC_OPTIMIZED_OUT:
-	  fputs_filtered ("optimized out", outfile);
-	  break;
-	case LOC_COMPUTED:
-	  fputs_filtered ("computed at runtime", outfile);
-	  break;
-	default:
-	  fputs_filtered ("<invalid location>", outfile);
-	  break;
-	}
-      fputs_filtered (", ", outfile);
-      fputs_filtered (paddress (gdbarch, SYMBOL_VALUE_ADDRESS (*p)), outfile);
-      fprintf_filtered (outfile, "\n");
-      p++;
-    }
-}
-
 void
 maintenance_print_msymbols (char *args, int from_tty)
 {
@@ -902,7 +673,8 @@ maintenance_print_msymbols (char *args, int from_tty)
 
   if (args == NULL)
     {
-      error (_("print-msymbols takes an output file name and optional symbol file name"));
+      error (_("print-msymbols takes an output file "
+	       "name and optional symbol file name"));
     }
   argv = gdb_buildargv (args);
   cleanups = make_cleanup_freeargv (argv);
@@ -910,7 +682,7 @@ maintenance_print_msymbols (char *args, int from_tty)
   if (argv[0] != NULL)
     {
       filename = argv[0];
-      /* If a second arg is supplied, it is a source file name to match on */
+      /* If a second arg is supplied, it is a source file name to match on.  */
       if (argv[1] != NULL)
 	{
 	  symname = xfullpath (argv[1]);
@@ -931,8 +703,8 @@ maintenance_print_msymbols (char *args, int from_tty)
   immediate_quit++;
   ALL_PSPACES (pspace)
     ALL_PSPACE_OBJFILES (pspace, objfile)
-      if (symname == NULL
-	  || (!stat (objfile->name, &obj_st) && sym_st.st_ino == obj_st.st_ino))
+      if (symname == NULL || (!stat (objfile->name, &obj_st)
+			      && sym_st.st_ino == obj_st.st_ino))
 	dump_msymbols (objfile, outfile);
   immediate_quit--;
   fprintf_filtered (outfile, "\n\n");
@@ -998,12 +770,15 @@ maintenance_info_symtabs (char *regexp, int from_tty)
 			       symtab->dirname ? symtab->dirname : "(null)");
 	      printf_filtered ("	  fullname %s\n",
 			       symtab->fullname ? symtab->fullname : "(null)");
-	      printf_filtered ("	  blockvector ((struct blockvector *) %s)%s\n",
+	      printf_filtered ("	  "
+			       "blockvector ((struct blockvector *) %s)%s\n",
 			       host_address_to_string (symtab->blockvector),
 			       symtab->primary ? " (primary)" : "");
-	      printf_filtered ("	  linetable ((struct linetable *) %s)\n",
+	      printf_filtered ("	  "
+			       "linetable ((struct linetable *) %s)\n",
 			       host_address_to_string (symtab->linetable));
-	      printf_filtered ("	  debugformat %s\n", symtab->debugformat);
+	      printf_filtered ("	  debugformat %s\n",
+			       symtab->debugformat);
 	      printf_filtered ("	}\n");
 	    }
 	}
@@ -1012,194 +787,6 @@ maintenance_info_symtabs (char *regexp, int from_tty)
         printf_filtered ("}\n");
     }
 }
-
-
-/* List all the partial symbol tables whose names match REGEXP (optional).  */
-void
-maintenance_info_psymtabs (char *regexp, int from_tty)
-{
-  struct program_space *pspace;
-  struct objfile *objfile;
-
-  if (regexp)
-    re_comp (regexp);
-
-  ALL_PSPACES (pspace)
-    ALL_PSPACE_OBJFILES (pspace, objfile)
-    {
-      struct gdbarch *gdbarch = get_objfile_arch (objfile);
-      struct partial_symtab *psymtab;
-
-      /* We don't want to print anything for this objfile until we
-         actually find a symtab whose name matches.  */
-      int printed_objfile_start = 0;
-
-      ALL_OBJFILE_PSYMTABS (objfile, psymtab)
-	{
-	  QUIT;
-
-	  if (! regexp
-	      || re_exec (psymtab->filename))
-	    {
-	      if (! printed_objfile_start)
-		{
-		  printf_filtered ("{ objfile %s ", objfile->name);
-		  wrap_here ("  ");
-		  printf_filtered ("((struct objfile *) %s)\n", 
-				   host_address_to_string (objfile));
-		  printed_objfile_start = 1;
-		}
-
-	      printf_filtered ("  { psymtab %s ", psymtab->filename);
-	      wrap_here ("    ");
-	      printf_filtered ("((struct partial_symtab *) %s)\n", 
-			       host_address_to_string (psymtab));
-
-	      printf_filtered ("    readin %s\n",
-			       psymtab->readin ? "yes" : "no");
-	      printf_filtered ("    fullname %s\n",
-			       psymtab->fullname ? psymtab->fullname : "(null)");
-	      printf_filtered ("    text addresses ");
-	      fputs_filtered (paddress (gdbarch, psymtab->textlow),
-			      gdb_stdout);
-	      printf_filtered (" -- ");
-	      fputs_filtered (paddress (gdbarch, psymtab->texthigh),
-			      gdb_stdout);
-	      printf_filtered ("\n");
-	      printf_filtered ("    globals ");
-	      if (psymtab->n_global_syms)
-		{
-		  printf_filtered ("(* (struct partial_symbol **) %s @ %d)\n",
-				   host_address_to_string (psymtab->objfile->global_psymbols.list
-				    + psymtab->globals_offset),
-				   psymtab->n_global_syms);
-		}
-	      else
-		printf_filtered ("(none)\n");
-	      printf_filtered ("    statics ");
-	      if (psymtab->n_static_syms)
-		{
-		  printf_filtered ("(* (struct partial_symbol **) %s @ %d)\n",
-				   host_address_to_string (psymtab->objfile->static_psymbols.list
-				    + psymtab->statics_offset),
-				   psymtab->n_static_syms);
-		}
-	      else
-		printf_filtered ("(none)\n");
-	      printf_filtered ("    dependencies ");
-	      if (psymtab->number_of_dependencies)
-		{
-		  int i;
-
-		  printf_filtered ("{\n");
-		  for (i = 0; i < psymtab->number_of_dependencies; i++)
-		    {
-		      struct partial_symtab *dep = psymtab->dependencies[i];
-
-		      /* Note the string concatenation there --- no comma.  */
-		      printf_filtered ("      psymtab %s "
-				       "((struct partial_symtab *) %s)\n",
-				       dep->filename, 
-				       host_address_to_string (dep));
-		    }
-		  printf_filtered ("    }\n");
-		}
-	      else
-		printf_filtered ("(none)\n");
-	      printf_filtered ("  }\n");
-	    }
-	}
-
-      if (printed_objfile_start)
-        printf_filtered ("}\n");
-    }
-}
-
-
-/* Check consistency of psymtabs and symtabs.  */
-
-void
-maintenance_check_symtabs (char *ignore, int from_tty)
-{
-  struct symbol *sym;
-  struct partial_symbol **psym;
-  struct symtab *s = NULL;
-  struct partial_symtab *ps;
-  struct blockvector *bv;
-  struct objfile *objfile;
-  struct block *b;
-  int length;
-
-  ALL_PSYMTABS (objfile, ps)
-  {
-    struct gdbarch *gdbarch = get_objfile_arch (objfile);
-    s = PSYMTAB_TO_SYMTAB (ps);
-    if (s == NULL)
-      continue;
-    bv = BLOCKVECTOR (s);
-    b = BLOCKVECTOR_BLOCK (bv, STATIC_BLOCK);
-    psym = ps->objfile->static_psymbols.list + ps->statics_offset;
-    length = ps->n_static_syms;
-    while (length--)
-      {
-	sym = lookup_block_symbol (b, SYMBOL_LINKAGE_NAME (*psym),
-				   NULL, SYMBOL_DOMAIN (*psym));
-	if (!sym)
-	  {
-	    printf_filtered ("Static symbol `");
-	    puts_filtered (SYMBOL_LINKAGE_NAME (*psym));
-	    printf_filtered ("' only found in ");
-	    puts_filtered (ps->filename);
-	    printf_filtered (" psymtab\n");
-	  }
-	psym++;
-      }
-    b = BLOCKVECTOR_BLOCK (bv, GLOBAL_BLOCK);
-    psym = ps->objfile->global_psymbols.list + ps->globals_offset;
-    length = ps->n_global_syms;
-    while (length--)
-      {
-	sym = lookup_block_symbol (b, SYMBOL_LINKAGE_NAME (*psym),
-				   NULL, SYMBOL_DOMAIN (*psym));
-	if (!sym)
-	  {
-	    printf_filtered ("Global symbol `");
-	    puts_filtered (SYMBOL_LINKAGE_NAME (*psym));
-	    printf_filtered ("' only found in ");
-	    puts_filtered (ps->filename);
-	    printf_filtered (" psymtab\n");
-	  }
-	psym++;
-      }
-    if (ps->texthigh < ps->textlow)
-      {
-	printf_filtered ("Psymtab ");
-	puts_filtered (ps->filename);
-	printf_filtered (" covers bad range ");
-	fputs_filtered (paddress (gdbarch, ps->textlow), gdb_stdout);
-	printf_filtered (" - ");
-	fputs_filtered (paddress (gdbarch, ps->texthigh), gdb_stdout);
-	printf_filtered ("\n");
-	continue;
-      }
-    if (ps->texthigh == 0)
-      continue;
-    if (ps->textlow < BLOCK_START (b) || ps->texthigh > BLOCK_END (b))
-      {
-	printf_filtered ("Psymtab ");
-	puts_filtered (ps->filename);
-	printf_filtered (" covers ");
-	fputs_filtered (paddress (gdbarch, ps->textlow), gdb_stdout);
-	printf_filtered (" - ");
-	fputs_filtered (paddress (gdbarch, ps->texthigh), gdb_stdout);
-	printf_filtered (" but symtab covers only ");
-	fputs_filtered (paddress (gdbarch, BLOCK_START (b)), gdb_stdout);
-	printf_filtered (" - ");
-	fputs_filtered (paddress (gdbarch, BLOCK_END (b)), gdb_stdout);
-	printf_filtered ("\n");
-      }
-  }
-}
 
 
 /* Return the nexting depth of a block within other blocks in its symtab.  */
@@ -1208,6 +795,7 @@ static int
 block_depth (struct block *block)
 {
   int i = 0;
+
   while ((block = BLOCK_SUPERBLOCK (block)) != NULL)
     {
       i++;
@@ -1216,36 +804,7 @@ block_depth (struct block *block)
 }
 
 
-/* Increase the space allocated for LISTP, which is probably
-   global_psymbols or static_psymbols. This space will eventually
-   be freed in free_objfile().  */
-
-void
-extend_psymbol_list (struct psymbol_allocation_list *listp,
-		     struct objfile *objfile)
-{
-  int new_size;
-  if (listp->size == 0)
-    {
-      new_size = 255;
-      listp->list = (struct partial_symbol **)
-	xmalloc (new_size * sizeof (struct partial_symbol *));
-    }
-  else
-    {
-      new_size = listp->size * 2;
-      listp->list = (struct partial_symbol **)
-	xrealloc ((char *) listp->list,
-		  new_size * sizeof (struct partial_symbol *));
-    }
-  /* Next assumes we only went one over.  Should be good if
-     program works correctly */
-  listp->next = listp->list + listp->size;
-  listp->size = new_size;
-}
-
-
-/* Do early runtime initializations. */
+/* Do early runtime initializations.  */
 void
 _initialize_symmisc (void)
 {
