@@ -1,9 +1,14 @@
 /* Trivial code used to test watchpoints in recursive code and 
    auto-deletion of watchpoints as they go out of scope.  */
 
+#ifdef PROTOTYPES
+static int
+recurse (int a)
+#else
 static int 
 recurse (a)
      int a;
+#endif
 {
   int b = 0;
 
@@ -15,11 +20,12 @@ recurse (a)
   return b;
 }
 
-main()
+int main()
 {
 #ifdef usestubs
   set_debug_traps();
   breakpoint();
 #endif
   recurse (10);
+  return 0;
 }
